@@ -1,3 +1,6 @@
+/*
+ * $Id: datablock.c,v 1.5 2014/04/05 06:17:08 markisch Exp $
+ */
 /* GNUPLOT - datablock.c */
 
 /*[
@@ -62,7 +65,6 @@
 #include "alloc.h"
 #include "command.h"
 #include "datablock.h"
-#include "datafile.h"
 #include "eval.h"
 #include "misc.h"
 #include "util.h"
@@ -91,7 +93,7 @@ datablock_command()
     int nlines;
     int nsize = 4;
     struct udvt_entry *datablock;
-    char *dataline = NULL;
+    char dataline[MAX_LINE_LEN+1];
 
     if (!isletter(c_token+1))
 	int_error(c_token, "illegal datablock name");
@@ -100,14 +102,14 @@ datablock_command()
     name = parse_datablock_name();
     datablock = add_udv_by_name(name);
 
-    if (!equals(c_token, "<<") || !isletter(c_token+1))
-	int_error(c_token, "data block name must be followed by << EODmarker");
-
-    if (datablock->udv_value.type != NOTDEFINED)
+    if (!datablock->udv_undef)
 	gpfree_datablock(&datablock->udv_value);
+    datablock->udv_undef = FALSE;
     datablock->udv_value.type = DATABLOCK;
     datablock->udv_value.v.data_array = NULL;
 
+    if (!equals(c_token, "<<") || !isletter(c_token+1))
+	int_error(c_token, "data block name must be followed by << EODmarker");
     c_token++;
     eod = (char *) gp_alloc(token[c_token].length +2, "datablock");
     copy_str(&eod[0], c_token, token[c_token].length + 2);
@@ -117,7 +119,7 @@ datablock_command()
     fin = (lf_head == NULL) ? stdin : lf_head->fp;
     if (!fin)
 	int_error(NO_CARET,"attempt to define data block from invalid context");
-    for (nlines = 0; (dataline = df_fgets(fin)); nlines++) {
+    for (nlines = 0; fgets(dataline, MAX_LINE_LEN, fin); nlines++) {
 	int n;
 
 	if (!strncmp(eod, dataline, strlen(eod)))
@@ -172,7 +174,7 @@ get_datablock(char *name)
     struct udvt_entry *datablock;
 
     datablock = get_udv_by_name(name);
-    if (!datablock || datablock->udv_value.type != DATABLOCK
+    if (!datablock || datablock->udv_undef
     ||  datablock->udv_value.v.data_array == NULL)
 	int_error(NO_CARET,"no datablock named %s",name);
 
@@ -195,39 +197,31 @@ gpfree_datablock(struct value *datablock_value)
     datablock_value->v.data_array = NULL;
 }
 
-/* count number of lines in a datablock */
-int
-datablock_size(struct value *datablock_value)
+
+/* resize or allocate a datablock; allocate memory in chuncks */
+static int
+enlarge_datablock(struct value *datablock_value, int extra)
 {
     char **dataline;
     int nlines = 0;
+    int osize, nsize;
+    const int blocksize = 512;
 
+    /* count number of lines in datablock */
     dataline = datablock_value->v.data_array;
     if (dataline) {
 	while (*dataline++)
 	    nlines++;
     }
-    return nlines;
-}
-
-/* resize or allocate a datablock; allocate memory in chunks */
-static int
-enlarge_datablock(struct value *datablock_value, int extra)
-{
-    int osize, nsize;
-    const int blocksize = 512;
-    int nlines = datablock_size(datablock_value);
 
     /* reserve space in multiples of blocksize */
     osize = ((nlines+1 + blocksize-1) / blocksize) * blocksize; 
     nsize = ((nlines+1 + extra + blocksize-1) / blocksize) * blocksize;
 
     /* only resize if necessary */
-    if ((osize != nsize) || (extra == 0) || (nlines == 0)) {
+    if ((osize != nsize) || (extra == 0) || (nlines == 0))
 	datablock_value->v.data_array =
 	    (char **) gp_realloc(datablock_value->v.data_array,  nsize * sizeof(char *), "resize_datablock");
-	datablock_value->v.data_array[nlines] = NULL;
-    }
 
     return nlines;
 }
@@ -241,42 +235,3 @@ append_to_datablock(struct value *datablock_value, const char *line)
     datablock_value->v.data_array[nlines] = (char *) line;
     datablock_value->v.data_array[nlines + 1] = NULL;
 }
-
-
-/* append multiple lines which are separated by linebreaks to a datablock */
-void
-append_multiline_to_datablock(struct value *datablock_value, const char *lines)
-{
-    char * l = (char *) lines;
-    char * p;
-    TBOOLEAN inquote = FALSE;
-    TBOOLEAN escaped = FALSE;
-
-    /* handle lines with line-breaks, one at a time;
-       take care of quoted strings
-     */
-    p = l;
-    while (*p != NUL) {
-	if (*p == '\'' && !escaped)
-	    inquote = !inquote;
-	else if (*p == '\\' && !escaped)
-	    escaped = TRUE;
-	else if (*p == '\n' && !inquote) {
-	    *p = NUL;
-	    append_to_datablock(datablock_value, strdup(l));
-	    l = p + 1;
-	} else
-	    escaped = FALSE;
-        p++;
-    }
-    if (l == lines) {
-	/* no line-breaks, just a single line */
-	append_to_datablock(datablock_value, l);
-    } else {
-	if (strlen(l) > 0) /* remainder after last line-break */
-	    append_to_datablock(datablock_value, strdup(l));
-	/* we allocated new sub-strings, free the original */
-	free((char *) lines);
-    }
-}
-
