@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: fit.c,v 1.145.2.11 2015/06/07 15:19:59 sfeam Exp $"); }
+#endif
+
 /*  NOTICE: Change of Copyright Status
  *
  *  The author of this module, Carsten Grammes, has expressed in
@@ -35,6 +39,9 @@
  * drd: allow 3d fitting. Data value is now called fit_z internally,
  * ie a 2d fit is z vs x, and a 3d fit is z vs x and y.
  *
+ * Lars Hecking : review update command, for VMS in particular, where
+ * it is not necessary to rename the old file.
+ *
  * HBB, 971023: lifted fixed limit on number of datapoints, and number
  * of parameters.
  *
@@ -93,7 +100,6 @@
 #include "alloc.h"
 #include "axis.h"
 #include "command.h"
-#include "datablock.h"
 #include "datafile.h"
 #include "eval.h"
 #include "gp_time.h"
@@ -109,9 +115,9 @@
 
 /* Just temporary */
 #if defined(VA_START) && defined(STDC_HEADERS)
-static void Dblfn(const char *fmt, ...);
+static void Dblfn __PROTO((const char *fmt, ...));
 #else
-static void Dblfn();
+static void Dblfn __PROTO(());
 #endif
 #define Dblf  Dblfn
 #define Dblf2 Dblfn
@@ -123,9 +129,10 @@ static void Dblfn();
 # include <io.h>
 # include <conio.h>
 # include <dos.h>
+#elif !defined(VMS)
+#  include <fcntl.h>
 #endif
-#ifdef _WIN32
-# include <fcntl.h>
+#ifdef WIN32
 # include "win/winmain.h"
 #endif
 
@@ -206,7 +213,6 @@ static double lambda_up_factor = LAMBDA_UP_FACTOR;
 static const char fitlogfile_default[] = "fit.log";
 static const char GNUFITLOG[] = "FIT_LOG";
 static FILE *log_f = NULL;
-static FILE *via_f = NULL;
 static TBOOLEAN fit_show_lambda = TRUE;
 static const char *GP_FIXED = "# FIXED";
 static const char *FITSCRIPT = "FIT_SCRIPT";
@@ -231,7 +237,6 @@ static TBOOLEAN user_stop = FALSE;
 static double *scale_params = 0; /* scaling values for parameters */
 static struct udft_entry func;
 static fixstr *par_name;
-static t_value **par_udv;	/* array of pointers to the "via" variables */
 
 static fixstr *last_par_name = NULL;
 static int last_num_params = 0;
@@ -248,35 +253,37 @@ static udvt_entry *fit_dummy_udvs[MAX_NUM_VAR];
 			 internal Prototypes
 *****************************************************************/
 
-#if !defined(_WIN32) || defined(WGP_CONSOLE)
-static RETSIGTYPE ctrlc_handle(int an_int);
+#if !defined(WIN32) || defined(WGP_CONSOLE)
+static RETSIGTYPE ctrlc_handle __PROTO((int an_int));
 #endif
-static void ctrlc_setup(void);
-static marq_res_t marquardt(double a[], double **alpha, double *chisq, double *lambda);
+static void ctrlc_setup __PROTO((void));
+static marq_res_t marquardt __PROTO((double a[], double **alpha, double *chisq,
+				     double *lambda));
 static void analyze(double a[], double **alpha, double beta[],
 				 double *chisq, double **deriv);
-static void calculate(double *zfunc, double **dzda, double a[]);
+static void calculate __PROTO((double *zfunc, double **dzda, double a[]));
 static void calc_derivatives(const double *par, double *data, double **deriv);
-static TBOOLEAN fit_interrupt(void);
-static TBOOLEAN regress(double a[]);
+static TBOOLEAN fit_interrupt __PROTO((void));
+static TBOOLEAN regress __PROTO((double a[]));
 static void regress_init(void);
 static void regress_finalize(int iter, double chisq, double last_chisq, double lambda, double **covar);
 static void fit_show(int i, double chisq, double last_chisq, double *a,
                           double lambda, FILE * device);
 static void fit_show_brief(int iter, double chisq, double last_chisq, double *parms,
                           double lambda, FILE * device);
-static void show_results(double chisq, double last_chisq, double* a, double* dpar, double** corel);
-static void log_axis_restriction(FILE *log_f, int param,
-			    double min, double max, int autoscale, char *name);
+static void show_results __PROTO((double chisq, double last_chisq, double* a, double* dpar, double** corel));
+static void log_axis_restriction __PROTO((FILE *log_f, int param,
+			    double min, double max, int autoscale, char *name));
 static void print_function_definitions(struct at_type *at, FILE * device);
-static TBOOLEAN is_empty(char *s);
-static intgr_t getivar(const char *varname);
-static double getdvar(const char *varname);
-static double createdvar(char *varname, double value);
-static void setvar(char *varname, double value);
-static void setvarerr(char *varname, double value);
+static TBOOLEAN is_empty __PROTO((char *s));
+static int getivar __PROTO((const char *varname));
+static double getdvar __PROTO((const char *varname));
+static double createdvar __PROTO((char *varname, double value));
+static void setvar __PROTO((char *varname, double value));
+static void setvarerr __PROTO((char *varname, double value));
 static void setvarcovar(char *varname1, char *varname2, double value);
-static char *get_next_word(char **s, char *subst);
+static char *get_next_word __PROTO((char **s, char *subst));
+static void backup_file __PROTO((char *, const char *));
 
 
 /*****************************************************************
@@ -301,7 +308,7 @@ wri_to_fil_last_fit_cmd(FILE *fp)
     This is called when a SIGINT occurs during fit
 *****************************************************************/
 
-#if !defined(_WIN32) || defined(WGP_CONSOLE)
+#if !defined(WIN32) || defined(WGP_CONSOLE)
 static RETSIGTYPE
 ctrlc_handle(int an_int)
 {
@@ -328,7 +335,7 @@ ctrlc_setup()
  *
  *  I hope that other OSes do it better, if not... add #ifdefs :-(
  */
-#if (defined(__EMX__) || !defined(MSDOS)) && (!defined(_WIN32) || defined(WGP_CONSOLE))
+#if (defined(__EMX__) || !defined(MSDOS)) && (!defined(WIN32) || defined(WGP_CONSOLE))
     (void) signal(SIGINT, (sigfunc) ctrlc_handle);
 #endif
 }
@@ -339,7 +346,8 @@ ctrlc_setup()
 *****************************************************************/
 #if defined(MSDOS)
 
-int getchx(void);
+/* HBB 980317: added a prototype... */
+int getchx __PROTO((void));
 
 int
 getchx()
@@ -373,10 +381,6 @@ error_ex(int t_num, const char *str, ...)
 	fclose(log_f);
 	log_f = NULL;
     }
-    if (via_f) {
-	fclose(via_f);
-	via_f = NULL;
-    }
     free(fit_x);
     free(fit_z);
     free(err_data);
@@ -396,12 +400,6 @@ error_ex(int t_num, const char *str, ...)
     /* restore original SIGINT function */
     interrupt_setup();
 
-    /* FIXME: It would be nice to exit the "fit" command non-fatally, */
-    /* so that the script who called it can recover and continue.     */
-    /* int_error() makes that impossible.  But if we use int_warn()   */
-    /* instead the program tries to continue _inside_ the fit, which  */
-    /* generally then dies on some more serious error.                */
-
     /* exit via int_error() so that it can clean up state variables */
     int_error(t_num, buf);
 }
@@ -641,22 +639,19 @@ call_gnuplot(const double *par, double *data)
 
     /* set parameters first */
     for (i = 0; i < num_params; i++)
-	Gcomplex(par_udv[i], par[i] * scale_params[i], 0.0);
+	setvar(par_name[i], par[i] * scale_params[i]);
 
     for (i = 0; i < num_data; i++) {
 	/* calculate fit-function value */
 	/* initialize extra dummy variables from the corresponding
 	 actual variables, if any. */
 	for (j = 0; j < MAX_NUM_VAR; j++) {
-	    double dummy_value;
 	    struct udvt_entry *udv = fit_dummy_udvs[j];
 	    if (!udv)
 		int_error(NO_CARET, "Internal error: lost a dummy parameter!");
-	    if (udv->udv_value.type == CMPLX || udv->udv_value.type == INTGR)
-		dummy_value = real(&(udv->udv_value));
-	    else
-		dummy_value = 0.0;
-	    Gcomplex(&func.dummy_values[j], dummy_value, 0.0);
+	    Gcomplex(&func.dummy_values[j],
+	             udv->udv_undef ? 0 : real(&(udv->udv_value)),
+	             0.0);
 	}
 	/* set actual dummy variables from file data */
 	for (j = 0; j < num_indep; j++)
@@ -664,7 +659,8 @@ call_gnuplot(const double *par, double *data)
 	             fit_x[i * num_indep + j], 0.0);
 	evaluate_at(func.at, &v);
 
-	if (undefined || isnan(real(&v))) {
+	data[i] = real(&v);
+	if (undefined || isnan(data[i])) {
 	    /* Print useful info on undefined-function error. */
 	    Dblf("\nCurrent data point\n");
 	    Dblf("=========================\n");
@@ -683,8 +679,6 @@ call_gnuplot(const double *par, double *data)
 		Eex("Function evaluation yields NaN (\"not a number\")");
 	    }
 	}
-
-	data[i] = real(&v);
     }
 }
 
@@ -702,7 +696,7 @@ calc_derivatives(const double *par, double *data, double **deriv)
 
     /* set parameters first */
     for (i = 0; i < num_params; i++)
-	Gcomplex(par_udv[i], par[i] * scale_params[i], 0.0);
+	setvar(par_name[i], par[i] * scale_params[i]);
 
     for (i = 0; i < num_data; i++) { /* loop over data points */
 	for (j = 0, m = 0; j < num_indep; j++) { /* loop over indep. variables */
@@ -749,7 +743,7 @@ fit_interrupt()
 {
     while (TRUE) {
 	fputs("\n\n(S)top fit, (C)ontinue, (E)xecute FIT_SCRIPT:  ", STANDARD);
-#ifdef _WIN32
+#ifdef WIN32
 	WinRaiseConsole();
 #endif
 	switch (getchar()) {
@@ -769,14 +763,16 @@ fit_interrupt()
 	case 'e':
 	case 'E':{
 		int i;
-		const char *tmp = getfitscript();
+		char *tmp;
 
+		tmp = getfitscript();
 		fprintf(STANDARD, "executing: %s\n", tmp);
 		/* FIXME: Shouldn't we also set FIT_STDFIT etc? */
 		/* set parameters visible to gnuplot */
 		for (i = 0; i < num_params; i++)
-		    Gcomplex(par_udv[i], a[i] * scale_params[i], 0.0);
+		    setvar(par_name[i], a[i] * scale_params[i]);
 		do_string(tmp);
+		free(tmp);
 	    }
 	}
     }
@@ -787,17 +783,17 @@ fit_interrupt()
 /*****************************************************************
     determine current setting of FIT_SCRIPT
 *****************************************************************/
-const char *
+char *
 getfitscript(void)
 {
     char *tmp;
 
     if (fit_script != NULL)
-	return fit_script;
+	return gp_strdup(fit_script);
     if ((tmp = getenv(FITSCRIPT)) != NULL)
-	return tmp;
+	return gp_strdup(tmp);
     else
-	return DEFAULT_CMD;
+	return gp_strdup(DEFAULT_CMD);
 }
 
 
@@ -811,6 +807,7 @@ regress_init(void)
 
     /* Reset flag describing fit result status */
     v = add_udv_by_name("FIT_CONVERGED");
+    v->udv_undef = FALSE;
     Ginteger(&v->udv_value, 0);
 
     /* Ctrl-C now serves as Hotkey */
@@ -876,6 +873,7 @@ regress_finalize(int iter, double chisq, double last_chisq, double lambda, doubl
     } else {
 	Dblf2("\nAfter %d iterations the fit converged.\n", iter);
 	v = add_udv_by_name("FIT_CONVERGED");
+	v->udv_undef = FALSE;
 	Ginteger(&v->udv_value, 1);
     }
 
@@ -887,21 +885,26 @@ regress_finalize(int iter, double chisq, double last_chisq, double lambda, doubl
 
     /* Export these to user-accessible variables */
     v = add_udv_by_name("FIT_NDF");
+    v->udv_undef = FALSE;
     Ginteger(&v->udv_value, ndf);
     v = add_udv_by_name("FIT_STDFIT");
+    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, stdfit, 0);
     v = add_udv_by_name("FIT_WSSR");
+    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, chisq, 0);
     v = add_udv_by_name("FIT_P");
+    v->udv_undef = FALSE;
     Gcomplex(&v->udv_value, pvalue, 0);
     v = add_udv_by_name("FIT_NITER");
+    v->udv_undef = FALSE;
     Ginteger(&v->udv_value, niter);    
 
     /* Save final parameters. Depending on the backend and
        its internal state, the last call_gnuplot may not have been
        at the minimum */
     for (i = 0; i < num_params; i++)
-	Gcomplex(par_udv[i], a[i] * scale_params[i], 0.0);
+	setvar(par_name[i], a[i] * scale_params[i]);
 
     /* Set error and covariance variables to zero, 
        thus making sure they are created. */
@@ -911,7 +914,7 @@ regress_finalize(int iter, double chisq, double last_chisq, double lambda, doubl
     }
     if (fit_covarvariables) {
 	/* first, remove all previous covariance variables */
-	del_udv_by_name("FIT_COV_", TRUE);
+	del_udv_by_name("FIT_COV_*", TRUE);
 	for (i = 0; i < num_params; i++) {
 	    for (j = 0; j < i; j++) {
 		setvarcovar(par_name[i], par_name[j], 0.0);
@@ -1005,7 +1008,7 @@ regress_check_stop(int iter, double chisq, double last_chisq, double lambda)
 	ctrlc_flag = TRUE;
     }
 #endif
-#ifdef _WIN32
+#ifdef WIN32
     /* This call makes the Windows GUI functional during fits.
        Pressing Ctrl-Break now finally has an effect. */
     WinMessageLoop();
@@ -1357,7 +1360,7 @@ get_next_word(char **s, char *subst)
 	tmp++;
     if (*tmp == '\n' || *tmp == '\r' || *tmp == '\0')	/* not found */
 	return NULL;
-    if ((*s = strpbrk(tmp, " =\t\n\r[")) == NULL)
+    if ((*s = strpbrk(tmp, " =\t\n\r")) == NULL)
 	*s = tmp + strlen(tmp);
     *subst = **s;
     *(*s)++ = '\0';
@@ -1382,13 +1385,7 @@ init_fit()
 static void
 setvar(char *varname, double data)
 {
-    char *c;
-
-    /* Sanitize name to remove square brackets from array variables */
-    for (c = varname; *c; c++) {
-	if (*c == '[' || *c == ']')
-	    *c = '_';
-    }
+    /* Despite its name it is actually usable for any variable. */
     fill_gpval_float(varname, data);
 }
 
@@ -1428,12 +1425,12 @@ setvarcovar(char *varname1, char *varname2, double value)
 /*****************************************************************
     Get integer variable value
 *****************************************************************/
-static intgr_t
+static int
 getivar(const char *varname)
 {
     struct udvt_entry * v = get_udv_by_name((char *)varname);
-    if ((v != NULL) && (v->udv_value.type != NOTDEFINED))
-	return (intgr_t)real(&(v->udv_value));
+    if ((v != NULL) && (!v->udv_undef))
+	return real_int(&(v->udv_value));
     else
 	return 0;
 }
@@ -1446,7 +1443,7 @@ static double
 getdvar(const char *varname)
 {
     struct udvt_entry * v = get_udv_by_name((char *)varname);
-    if ((v != NULL) && (v->udv_value.type != NOTDEFINED))
+    if ((v != NULL) && (!v->udv_undef))
 	return real(&(v->udv_value));
     else
 	return 0;
@@ -1462,7 +1459,8 @@ static double
 createdvar(char *varname, double value)
 {
     struct udvt_entry *udv_ptr = add_udv_by_name((char *)varname);
-    if (udv_ptr->udv_value.type == NOTDEFINED) { /* new variable */
+    if (udv_ptr->udv_undef) { /* new variable */
+	udv_ptr->udv_undef = FALSE;
 	Gcomplex(&udv_ptr->udv_value, value, 0.0);
     } else if (udv_ptr->udv_value.type == INTGR) { /* convert to CMPLX */
 	Gcomplex(&udv_ptr->udv_value, (double) udv_ptr->udv_value.v.int_val, 0.0);
@@ -1471,6 +1469,244 @@ createdvar(char *varname, double value)
 }
 
 
+/* argument: char *fn */
+#define VALID_FILENAME(fn) ((fn) != NULL && (*fn) != '\0')
+
+/*****************************************************************
+    write the actual parameters to start parameter file
+*****************************************************************/
+void
+update(char *pfile, char *npfile)
+{
+    char ifilename[PATH_MAX];
+    char *ofilename;
+    TBOOLEAN createfile = FALSE;
+
+    if (existfile(pfile)) {
+	/* update pfile npfile:
+	   if npfile is a valid file name, take pfile as input file and
+	   npfile as output file
+	*/
+	if (VALID_FILENAME(npfile)) {
+	    safe_strncpy(ifilename, pfile, sizeof(ifilename));
+	    ofilename = npfile;
+	} else {
+#ifdef BACKUP_FILESYSTEM
+	    /* filesystem will keep original as previous version */
+	    safe_strncpy(ifilename, pfile, sizeof(ifilename));
+#else
+	    backup_file(ifilename, pfile);	/* will Eex if it fails */
+#endif
+	    ofilename = pfile;
+	}
+    } else {
+	/* input file does not exists; will create new file */
+	createfile = TRUE;
+	if (VALID_FILENAME(npfile))
+	    ofilename = npfile;
+	else
+	    ofilename = pfile;
+    }
+
+    if (createfile) {
+	/* The input file does not exists and--strictly speaking--there is
+	   nothing to 'update'.  Instead of bailing out we guess the intended use:
+	   We output all INTGR/CMPLX user variables and mark them as '# FIXED' if
+	   they were not used during the last fit command. */
+	struct udvt_entry *udv = first_udv;
+	FILE *nf;
+
+	if ((last_fit_command == NULL) || (strlen(last_fit_command) == 0)) {
+	    /* Technically, a prior fit command isn't really required.  But since
+	    all variables in the parameter file would be marked '# FIXED' in that
+	    case, it cannot be directly used in a subsequent fit command. */
+#if 1
+	    Eex2("'update' requires a prior 'fit' since the parameter file %s does not exist yet.", ofilename);
+#else
+	    fprintf(stderr, "'update' without a prior 'fit' and without a previous parameter file:\n");
+	    fprintf(stderr, " all variables will be marked '# FIXED'!\n");
+#endif
+	}
+
+	if (!(nf = fopen(ofilename, "w")))
+	    Eex2("new parameter file %s could not be created", ofilename);
+
+	fputs("# Parameter file created by 'update' from current variables\n", nf);
+	if ((last_fit_command != NULL) && (strlen(last_fit_command) > 0))
+	    fprintf(nf, "## %s\n", last_fit_command);
+
+	while (udv) {
+	    if ((strncmp(udv->udv_name, "GPVAL_", 6) == 0) ||
+	        (strncmp(udv->udv_name, "MOUSE_", 6) == 0) ||
+	        (strncmp(udv->udv_name, "FIT_", 4) == 0) ||
+	        (strcmp(udv->udv_name, "NaN") == 0) ||
+	        (strcmp(udv->udv_name, "pi") == 0)) {
+		/* skip GPVAL_, MOUSE_, FIT_ and builtin variables */
+		udv = udv->next_udv;
+		continue;
+	    }
+	    if (!udv->udv_undef &&
+	        ((udv->udv_value.type == INTGR) || (udv->udv_value.type == CMPLX))) {
+		int k;
+
+		/* ignore indep. variables */
+		for (k = 0; k < MAX_NUM_VAR; k++) {
+		    if (last_dummy_var[k] && strcmp(last_dummy_var[k], udv->udv_name) == 0)
+			break;
+		}
+		if (k != MAX_NUM_VAR) {
+		    udv = udv->next_udv;
+		    continue;
+		}
+
+		if (udv->udv_value.type == INTGR)
+		    fprintf(nf, "%-15s = %-22i", udv->udv_name, udv->udv_value.v.int_val);
+		else /* CMPLX */
+		    fprintf(nf, "%-15s = %-22s", udv->udv_name, num_to_str(udv->udv_value.v.cmplx_val.real));
+		/* mark variables not used for the last fit as fixed */
+		for (k = 0; k < last_num_params; k++) {
+		    if (strcmp(last_par_name[k], udv->udv_name) == 0)
+			break;
+		}
+		if (k == last_num_params)
+		    fprintf(nf, "   %s", GP_FIXED);
+		putc('\n', nf);
+	    }
+	    udv = udv->next_udv;
+	}
+
+	if (fclose(nf))
+	    Eex("I/O error during update");
+
+    } else { /* !createfile */
+
+	/* input file exists - this is the originally intended case of
+	   the update command: update an existing parameter file */
+	char sstr[256];
+	char *s = sstr;
+	char * fnam;
+	FILE *of, *nf;
+
+	if (!(of = loadpath_fopen(ifilename, "r")))
+	    Eex2("parameter file %s could not be read", ifilename);
+
+	if (!(nf = fopen(ofilename, "w")))
+	    Eex2("new parameter file %s could not be created", ofilename);
+
+	fnam = gp_basename(ifilename); /* strip off the path */
+	if (fnam == NULL)
+	    fnam = ifilename;
+
+	while (fgets(s = sstr, sizeof(sstr), of) != NULL) {
+	    char pname[64]; /* name of parameter */
+	    double pval;    /* parameter value */
+	    char tail[127]; /* trailing characters */
+	    char * tmp;
+	    char c;
+
+	    if (is_empty(s)) {
+		fputs(s, nf);	/* preserve comments */
+		continue;
+	    }
+	    if ((tmp = strchr(s, '#')) != NULL) {
+		safe_strncpy(tail, tmp, sizeof(tail));
+		*tmp = NUL;
+	    } else
+		strcpy(tail, "\n");
+
+	    tmp = get_next_word(&s, &c);
+	    if (!legal_identifier(tmp) || strlen(tmp) > MAX_ID_LEN) {
+		fclose(nf);
+		fclose(of);
+		Eex2("syntax error in parameter file %s", fnam);
+	    }
+	    safe_strncpy(pname, tmp, sizeof(pname));
+	    /* next must be '=' */
+	    if (c != '=') {
+		tmp = strchr(s, '=');
+		if (tmp == NULL) {
+		    fclose(nf);
+		    fclose(of);
+		    Eex2("syntax error in parameter file %s", fnam);
+		}
+		s = tmp + 1;
+	    }
+	    tmp = get_next_word(&s, &c);
+	    if (!sscanf(tmp, "%lf", &pval)) {
+		fclose(nf);
+		fclose(of);
+		Eex2("syntax error in parameter file %s", fnam);
+	    }
+	    if ((tmp = get_next_word(&s, &c)) != NULL) {
+		fclose(nf);
+		fclose(of);
+		Eex2("syntax error in parameter file %s", fnam);
+	    }
+
+	    /* now modify */
+	    pval = getdvar(pname);
+	    fprintf(nf, "%-15s = %-22s   %s", pname, num_to_str(pval), tail);
+	}
+
+	if (fclose(nf) || fclose(of))
+	    Eex("I/O error during update");
+    }
+}
+
+
+/*****************************************************************
+    Backup a file by renaming it to something useful. Return
+    the new name in tofile
+*****************************************************************/
+
+/* tofile must point to a char array[] or allocated data. See update() */
+
+static void
+backup_file(char *tofile, const char *fromfile)
+{
+#if defined(MSDOS) || defined(VMS)
+    char *tmpn;
+#endif
+
+/* first attempt, for all o/s other than MSDOS */
+
+#ifndef MSDOS
+    strcpy(tofile, fromfile);
+#ifdef VMS
+    /* replace all dots with _, since we will be adding the only
+     * dot allowed in VMS names
+     */
+    while ((tmpn = strchr(tofile, '.')) != NULL)
+	*tmpn = '_';
+#endif /*VMS */
+    strcat(tofile, BACKUP_SUFFIX);
+    if (rename(fromfile, tofile) == 0)
+	return;			/* hurrah */
+    if (existfile(tofile))
+	Eex2("The backup file %s already exists and will not be overwritten.", tofile);
+#endif
+
+#ifdef MSDOS
+    /* first attempt for msdos. */
+
+    /* Copy only the first 8 characters of the filename, to comply
+     * with the restrictions of FAT filesystems. */
+    safe_strncpy(tofile, fromfile, 8 + 1);
+
+    while ((tmpn = strchr(tofile, '.')) != NULL)
+	*tmpn = '_';
+
+    strcat(tofile, BACKUP_SUFFIX);
+
+    if (rename(fromfile, tofile) == 0)
+	return;			/* success */
+#endif /* MSDOS */
+
+    /* get here => rename failed. */
+    Eex3("Could not rename file %s to %s", fromfile, tofile);
+}
+
+
 /* Modified from save.c:save_range() */
 static void
 log_axis_restriction(FILE *log_f, int param, double min, double max, int autoscale, char *name)
@@ -1572,6 +1808,9 @@ print_function_definitions(struct at_type *at, FILE * device)
 void
 fit_command()
 {
+/* HBB 20000430: revised this completely, to make it more similar to
+ * what plot3drequest() does */
+
     /* Backwards compatibility - these were the default names in 4.4 and 4.6	*/
     static const char *dummy_old_default[5] = {"x","y","t","u","v"};
 
@@ -1591,24 +1830,19 @@ fit_command()
     static const int iz = MAX_NUM_VAR;
 
     int i, j;
-    double v[MAX_NUM_VAR+2];
+    double v[MAXDATACOLS];
     double tmpd;
     time_t timer;
     int token1, token2, token3;
-    int fit_token;
     char *tmp, *file_name;
-    char *viafile;
     TBOOLEAN zero_initial_value;
-    AXIS *fit_xaxis, *fit_yaxis, *fit_zaxis;
 
+    c_token++;
+
+    /* FIXME EAM - I don't understand what these are needed for. */
     x_axis = FIRST_X_AXIS;
     y_axis = FIRST_Y_AXIS;
     z_axis = FIRST_Z_AXIS;
-    fit_xaxis = &axis_array[FIRST_X_AXIS];
-    fit_yaxis = &axis_array[FIRST_Y_AXIS];
-    fit_zaxis = &axis_array[FIRST_Z_AXIS];
-
-    fit_token = c_token++;
 
     /* First look for a restricted fit range... */
     /* Start with the current range limits on variable 1 ("x"),
@@ -1616,37 +1850,37 @@ fit_command()
      * Historically variables 3-5 inherited the current range of t, u, and v
      * but no longer.  NB: THIS IS A CHANGE
      */
-    axis_init(fit_xaxis, 0);
-    axis_init(fit_yaxis, 0);
-    axis_init(fit_zaxis, 1);
+    AXIS_INIT3D(x_axis, 0, 0);
+    AXIS_INIT3D(y_axis, 0, 0);
+    AXIS_INIT3D(z_axis, 0, 1);
     for (i = 0; i < MAX_NUM_VAR+1; i++)
 	dummy_token[i] = -1;
-    range_min[0] = fit_xaxis->min;
-    range_max[0] = fit_xaxis->max;
-    range_autoscale[0] = fit_xaxis->autoscale;
-    range_min[1] = fit_yaxis->min;
-    range_max[1] = fit_yaxis->max;
-    range_autoscale[1] = fit_yaxis->autoscale;
+    range_min[0] = axis_array[x_axis].min;
+    range_max[0] = axis_array[x_axis].max;
+    range_autoscale[0] = axis_array[x_axis].autoscale;
+    range_min[1] = axis_array[y_axis].min;
+    range_max[1] = axis_array[y_axis].max;
+    range_autoscale[1] = axis_array[y_axis].autoscale;
     for (i = 2; i < MAX_NUM_VAR; i++) {
 	range_min[i] = VERYLARGE;
 	range_max[i] = -VERYLARGE;
 	range_autoscale[i] = AUTOSCALE_BOTH;
     }
-    range_min[iz] = fit_zaxis->min;
-    range_max[iz] = fit_zaxis->max;
-    range_autoscale[iz] = fit_zaxis->autoscale;
+    range_min[iz] = axis_array[z_axis].min;
+    range_max[iz] = axis_array[z_axis].max;
+    range_autoscale[iz] = axis_array[z_axis].autoscale;
 
     num_ranges = 0;
     while (equals(c_token, "[")) {
-	AXIS *scratch_axis = &axis_array[SAMPLE_AXIS];
 	if (i > MAX_NUM_VAR)
 	    Eexc(c_token, "too many range specifiers");
-	axis_init(scratch_axis, 1);
-	scratch_axis->linked_to_primary = NULL;
-	dummy_token[num_ranges] = parse_range(scratch_axis->index);
-	range_min[num_ranges] = scratch_axis->min;
-	range_max[num_ranges] = scratch_axis->max;
-	range_autoscale[num_ranges] = scratch_axis->autoscale;
+	/* NB: This has nothing really to do with the Z axis, we're */
+	/* just using that slot of the axis array as scratch space. */
+	AXIS_INIT3D(SECOND_Z_AXIS, 0, 1);
+	dummy_token[num_ranges] = parse_range(SECOND_Z_AXIS);
+	range_min[num_ranges] = axis_array[SECOND_Z_AXIS].min;
+	range_max[num_ranges] = axis_array[SECOND_Z_AXIS].max;
+	range_autoscale[num_ranges] = axis_array[SECOND_Z_AXIS].autoscale;
 	num_ranges++;
     }
 
@@ -1683,10 +1917,6 @@ fit_command()
     else
 	Eexc(token2, "missing filename or datablock");
 
-    /* We accept a datablock but not a voxel grid */
-    if (*file_name == '$' && !get_datablock(file_name))
-	int_error(c_token-1, "cannot fit voxel data");
-
     /* use datafile module to parse the datafile and qualifiers */
     df_set_plot_mode(MODE_QUERY);  /* Does nothing except for binary datafiles */
 
@@ -1697,7 +1927,7 @@ fit_command()
     /* the limit on parameters passed to a user-defined function.		*/
     /* I.e. we expect that MAXDATACOLS >= MAX_NUM_VAR + 2			*/
 
-    columns = df_open(file_name, MAX_NUM_VAR+2, NULL);
+    columns = df_open(file_name, MAXDATACOLS, NULL);
     if (columns < 0)
 	Eexc2(token2, "Can't read data from", file_name);
     free(file_name);
@@ -1705,8 +1935,8 @@ fit_command()
 	Eexc(c_token, "Need more than 1 input data column");
 
     /* Allow time data only on first two dimensions (x and y) */
-    df_axis[0] = FIRST_X_AXIS;
-    df_axis[1] = FIRST_Y_AXIS;
+    df_axis[0] = x_axis;
+    df_axis[1] = y_axis;
 
     /* BM: New options to distinguish fits with and without errors */
     /* reset error columns */
@@ -1781,7 +2011,7 @@ fit_command()
 	err_cols[iz] = TRUE;
 	c_token++;
     } else if (almost_equals(c_token, "xyerr$ors")) {
-	/* convenience alias, x:z:sx:sz (or x:y:sx:sy) */
+	/* convienience alias, x:z:sx:sz (or x:y:sx:sy) */
 	if ((columns != 0) && (columns != 4))
 	    Eexc(c_token, "xyerror requires exactly 4 columns");
 	num_indep = 1;
@@ -1832,7 +2062,7 @@ fit_command()
     if (num_ranges > num_indep+1)
 	Eexc2(dummy_token[num_ranges-1], "Too many range-specs for a %d-variable fit", num_indep);
     if (num_ranges == (num_indep + 1)) {
-	/* last range was actually for the independen variable */
+	/* last range was actually for the dependent variable */
 	range_min[iz] = range_min[num_indep];
 	range_max[iz] = range_max[num_indep];
 	range_autoscale[iz] = range_autoscale[num_indep];
@@ -1860,7 +2090,7 @@ fit_command()
     /* tsm patchset 230: new absolute convergence variable */
     FPRINTF((STANDARD, "epsilon_abs=%e\n", epsilon_abs));
 
-    /* maximum number of iterations */
+    /* HBB 970304: maxiter patch */
     maxiter = getivar(FITMAXITER);
     if (maxiter < 0)
 	maxiter = 0;
@@ -1902,15 +2132,11 @@ fit_command()
 	free(line);
 	for (i = 0; (i < num_indep) && (i < columns - 1); i++)
 	    fprintf(log_f, "%s:", c_dummy_var[i]);
-	fprintf(log_f, "z");
-	if (num_errors > 0) {
+	if (num_errors > 0)
 	    for (i = 0; (i < num_indep) && (i < columns - 1); i++)
 		if (err_cols[i])
-		    fprintf(log_f, ":s%s", c_dummy_var[i]);
-	    fprintf(log_f, ":s\n");
-	} else {
-	    fprintf(log_f, "\n");
-	}
+		    fprintf(log_f, "s%s:", c_dummy_var[i]);
+	fprintf(log_f, (num_errors == 0) ? "z\n" : "z:s\n");
     }
 
     /* report all range specs, starting with Z */
@@ -1942,12 +2168,19 @@ fit_command()
 
     while ((i = df_readline(v, num_indep + num_errors + 1)) != DF_EOF) {
         if (num_data >= max_data) {
-	    max_data *= 2;
-	    if (!redim_vec(&fit_x, max_data * num_indep) ||
-		!redim_vec(&fit_z, max_data) ||
-		!redim_vec(&err_data, max_data * GPMAX(num_errors, 1))) {
+	    /* increase max_data by factor of 1.5 */
+	    max_data = (max_data * 3) / 2;
+	    if (0
+		|| !redim_vec(&fit_x, max_data * num_indep)
+		|| !redim_vec(&fit_z, max_data)
+		|| !redim_vec(&err_data, max_data * GPMAX(num_errors, 1))
+		) {
 		/* Some of the reallocations went bad: */
 		Eex2("Out of memory in fit: too many datapoints (%d)?", max_data);
+	    } else {
+		/* Just so we know that the routine is at work: */
+		fprintf(STANDARD, "Max. number of data points scaled up to: %d\n",
+			max_data);
 	    }
 	} /* if (need to extend storage space) */
 
@@ -1972,8 +2205,6 @@ fit_command()
 	    continue;
 	case DF_COLUMN_HEADERS:
 	    continue;
-	case DF_FOUND_KEY_TITLE:
-	    continue;
 	case 0:
 	    Eex2("bad data on line %d of datafile", df_line_number);
 	    break;
@@ -1983,7 +2214,7 @@ fit_command()
 	    break;
 	default:	/* June 2013 - allow more than 7 data columns */
 	    if (i<0)
-		Eex("unexpected value returned by df_readline");
+		int_error(NO_CARET, "unexpected value returned by df_readline");
 	    break;
 	}
 	num_points++;
@@ -2112,38 +2343,36 @@ fit_command()
     }
 
     /* read in parameters */
-    max_params = MAX_PARAMS;
+    max_params = MAX_PARAMS;	/* HBB 971023: make this resizeable */
 
     if (!equals(c_token++, "via"))
 	Eexc(c_token, "Need via and either parameter list or file");
 
     /* allocate arrays for parameter values, names */
     a = vec(max_params);
-    par_name = (fixstr *) gp_alloc((max_params + 1) * sizeof(fixstr), "fit param name");
-    par_udv = gp_realloc(par_udv, (max_params + 1) * sizeof(t_value *), "fit param pointer");
+    par_name = (fixstr *) gp_alloc((max_params + 1) * sizeof(fixstr),
+				   "fit param");
     num_params = 0;
 
-    /*
-     * FIXME: This is all done by character-by-character inspection of the
-     * input line.  If it were wrapped in lf_push()/lf_pop() we could use
-     * the normal gnuplot parsing routines keyed off c_token.
-     */
-    if ((viafile = try_to_get_string())) {	/* It's a parameter *file* */
+    if (isstringvalue(c_token)) {	/* It's a parameter *file* */
 	TBOOLEAN fixed;
 	double tmp_par;
-	char c='\0', *s;
+	char c, *s;
 	char sstr[MAX_LINE_LEN + 1];
+	FILE *f;
 
-	if (!(via_f = loadpath_fopen(viafile, "r")))
-	    Eex2("could not read parameter-file \"%s\"", viafile);
+	static char *viafile = NULL;
+	free(viafile);			/* Free previous name, if any */
+	viafile = try_to_get_string();	/* Cannot fail since isstringvalue succeeded */
 	if (!fit_suppress_log)
 	    fprintf(log_f, "fitted parameters and initial values from file: %s\n\n", viafile);
-	free(viafile);			/* Free previous name, if any */
+	if (!(f = loadpath_fopen(viafile, "r")))
+	    Eex2("could not read parameter-file \"%s\"", viafile);
 
 	/* get parameters and values out of file and ignore fixed ones */
 
 	while (TRUE) {
-	    if (!fgets(s = sstr, sizeof(sstr), via_f))	/* EOF found */
+	    if (!fgets(s = sstr, sizeof(sstr), f))	/* EOF found */
 		break;
 	    if ((tmp = strstr(s, GP_FIXED)) != NULL) {	/* ignore fixed params */
 		*tmp = NUL;
@@ -2157,47 +2386,49 @@ fit_command()
 	    if (is_empty(s))
 		continue;
 	    tmp = get_next_word(&s, &c);
-	    if (!legal_identifier(tmp) || strlen(tmp) > MAX_ID_LEN)
+	    if (!legal_identifier(tmp) || strlen(tmp) > MAX_ID_LEN) {
+		(void) fclose(f);
 		Eex("syntax error in parameter file");
-	    if (c == '[') {
-		/* Special case: array element */
-		udvt_entry *udv = get_udv_by_name(tmp);
-		int index;
-		if (udv->udv_value.type != ARRAY)
-		    Eex("no such array");
-		if ((1 != sscanf(s, "%d]", &index))
-		||  (index <= 0 || index > udv->udv_value.v.value_array[0].v.int_val))
-		    Eex("bad array index");
-		snprintf(par_name[num_params], sizeof(par_name[0]), "%40.40s[%d]", tmp, (short)index);
-		par_udv[num_params] = &(udv->udv_value.v.value_array[index]);
-	    } else {
-		/* Normal case */
-		safe_strncpy(par_name[num_params], tmp, sizeof(fixstr));
-		par_udv[num_params] = &(add_udv_by_name(tmp)->udv_value);
 	    }
+	    safe_strncpy(par_name[num_params], tmp, sizeof(fixstr));
 	    /* next must be '=' */
 	    if (c != '=') {
 		tmp = strchr(s, '=');
-		if (tmp == NULL)
+		if (tmp == NULL) {
+		    (void) fclose(f);
 		    Eex("syntax error in parameter file");
+		}
 		s = tmp + 1;
 	    }
 	    tmp = get_next_word(&s, &c);
-	    if (sscanf(tmp, "%lf", &tmp_par) != 1)
+	    if (sscanf(tmp, "%lf", &tmp_par) != 1) {
+		(void) fclose(f);
 		Eex("syntax error in parameter file");
-	    Gcomplex(par_udv[num_params], tmp_par, 0.0);
-	    /* Fixed parameters are updated but not counted against num_params */
-	    if (!fixed) {
-		if (num_params >= max_params)
-		    Eex("too many fit parameters");
+	    }
+	    /* make fixed params visible to GNUPLOT */
+	    if (fixed) {
+		/* use parname as temp */
+		setvar(par_name[num_params], tmp_par);
+	    } else {
+		if (num_params >= max_params) {
+		    max_params = (max_params * 3) / 2;
+		    if (0
+			|| !redim_vec(&a, max_params)
+			|| !(par_name = (fixstr *) gp_realloc(par_name, (max_params + 1) * sizeof(fixstr), "fit param resize"))
+			) {
+			(void) fclose(f);
+			Eex("Out of memory in fit: too many parameters?");
+		    }
+		}
 		a[num_params++] = tmp_par;
 	    }
 
-	    if ((tmp = get_next_word(&s, &c)) != NULL)
-		Eex("syntax error in parameter file");
+	    if ((tmp = get_next_word(&s, &c)) != NULL) {
+		(void) fclose(f);
+		Eex2("syntax error in parameter file %s", viafile);
+	    }
 	}
-	(void) fclose(via_f);
-	via_f = NULL;
+	(void) fclose(f);
 
     } else {
 	/* not a string after via: it's a variable listing */
@@ -2207,36 +2438,19 @@ fit_command()
 	do {
 	    if (!isletter(c_token))
 		Eex("no parameter specified");
-
-	    if (num_params >= max_params)
-		Eex("too many fit parameters");
-
-	    if (equals(c_token+1, "[")) {
-		/* Special case:  via Array[n]
-		 *	created variables will be of the form Array_n_*
-		 */
-		udvt_entry *udv = add_udv(c_token);
-		int index;
-		if (udv->udv_value.type != ARRAY)
-		    Eexc(c_token, "No such array");
-		c_token += 2;
-		index = int_expression();
-		if (index <= 0 || index > udv->udv_value.v.value_array[0].v.int_val)
-		    Eexc(c_token, "array index out of range");
-		if (!equals(c_token, "]"))
-		    Eexc(c_token, "not an array index");
-		snprintf(par_name[num_params], sizeof(par_name[0]), "%s[%d]", udv->udv_name, index);
-		a[num_params] = real( &(udv->udv_value.v.value_array[index]) );
-		par_udv[num_params] = &(udv->udv_value.v.value_array[index]);
-	    } else {
-		/* Normal case: via param_name */
-		capture(par_name[num_params], c_token, c_token, (int) sizeof(par_name[0]));
-		/* create variable if it doesn't exist */
-		a[num_params] = createdvar(par_name[num_params], INITIAL_VALUE);
-		par_udv[num_params] = &(get_udv_by_name(par_name[num_params])->udv_value);
+	    capture(par_name[num_params], c_token, c_token, (int) sizeof(par_name[0]));
+	    if (num_params >= max_params) {
+		max_params = (max_params * 3) / 2;
+		if (0
+		    || !redim_vec(&a, max_params)
+		    || !(par_name = (fixstr *) gp_realloc(par_name, (max_params + 1) * sizeof(fixstr), "fit param resize"))
+		    ) {
+		    Eex("Out of memory in fit: too many parameters?");
+		}
 	    }
-
-	    num_params++;
+	    /* create variable if it doesn't exist */
+	    a[num_params] = createdvar(par_name[num_params], INITIAL_VALUE);
+	    ++num_params;
 	} while (equals(++c_token, ",") && ++c_token);
     }
 
@@ -2300,12 +2514,15 @@ fit_command()
 	free(last_dummy_var[i]);
 	last_dummy_var[i] = gp_strdup(c_dummy_var[i]);
     }
-    /* remember last fit command for 'save fit' */
-    /* FIXME: This breaks if there is a ; internal to the fit command */
+    /* remember last fit command for 'save' */
     free(last_fit_command);
-    last_fit_command = strdup(&gp_input_line[token[fit_token].start_index]);
-    if (strchr(last_fit_command,';'))
-	*strchr(last_fit_command,';') = '\0';
+    last_fit_command = strdup(gp_input_line);
+    for (i = 0; i < num_tokens; i++) {
+	if (equals(i,";")) {
+	    last_fit_command[token[i].start_index] = '\0';
+	    break;
+	}
+    }
     /* save fit command to user variable */
     fill_gpval_string("GPVAL_LAST_FIT", last_fit_command);
 }
@@ -2361,12 +2578,6 @@ getfitlogfile()
     if (fitlogfile == NULL) {
 	char *tmp = getenv(GNUFITLOG);	/* open logfile */
 
-	/* If GNUFITLOG is defined but null, do not write to log file */
-	if (tmp != NULL && *tmp == '\0') {
-	    fit_suppress_log = TRUE;
-	    return NULL;
-	}
-
 	if (tmp != NULL && *tmp != '\0') {
 	    char *tmp2 = tmp + (strlen(tmp) - 1);
 
@@ -2389,34 +2600,3 @@ getfitlogfile()
     }
     return logfile;
 }
-
-
-/*
- * replacement for "update", which is now deprecated.
- * write current value of parameters used in previous fit to a file.
- * That file can be used as an argument to 'via' in a subsequent fit command.
- */
-void
-save_fit(FILE *fp)
-{
-    struct udvt_entry *udv;
-    int k;
-
-    if ((last_fit_command == NULL) || (strlen(last_fit_command) == 0)) {
-	int_warn(NO_CARET, "no previous fit command");
-	return;
-    } else {
-	fputs("# ", fp);
-	fputs(last_fit_command, fp);
-	fputs("\n", fp);
-	udv = get_udv_by_name("FIT_STDFIT");
-	if (udv)
-	    fprintf(fp,"# final sum of squares of residuals : %g\n",
-			udv->udv_value.v.cmplx_val.real);
-    }
-
-    for (k = 0; k < last_num_params; k++)
-	fprintf(fp, "%-15s = %-22s\n", last_par_name[k], value_to_str(par_udv[k], FALSE));
-
-    return;
-}
