@@ -1,13 +1,15 @@
+/*
+ * $Id: libcerf.c,v 1.1 2013/07/13 05:52:43 sfeam Exp $
+ */
 /*
  * Complex error function (cerf) and related special functions from libcerf.
- * libcerf itself uses the C99 _Complex mechanism for describing complex
+ * libcerf itself uses the C99 _Complex mechanism for decribing complex
  * numbers.  This set of wrapper routines converts back and forth from 
  * gnuplot's own representation of complex numbers.
  * Ethan A Merritt - July 2013
  */
 
-#include "gp_types.h"
-
+#include "syscfg.h"
 #ifdef HAVE_LIBCERF
 #include <complex.h>	/* C99 _Complex */
 #include <cerf.h>	/* libcerf library header */
