@@ -70,7 +70,6 @@ void QtGnuplotEnhanced::addText(const QString& fontName, double fontSize,
 	}
 	font.setStyle(fontStyle);
 	font.setWeight(fontWeight);
-	font.setStyleStrategy(QFont::ForceOutline);	// pcf fonts die if rotated
 	QtGnuplotEnhancedFragment* item = new QtGnuplotEnhancedFragment(font, text, this);
 	item->setPos(m_currentPos + QPointF(0., -base));
 	if (showFlag)
@@ -80,12 +79,12 @@ void QtGnuplotEnhanced::addText(const QString& fontName, double fontSize,
 
 	if (overprint == 2)                         // Overprint
 	{
-		item->setPos(QPointF((m_overprintPos + m_currentPos.x())/2. - item->width()/2., -base));
+		item->setPos(QPointF((m_overprintPos + m_currentPos.x())/2. - (item->boundingRect().right() + item->boundingRect().left())/2., -base));
 		m_overprintMark = false;
 	}
 
 	if (widthFlag && (overprint != 2))
-		m_currentPos += QPointF(item->width(), 0.);
+		m_currentPos += QPointF(item->boundingRect().right(), 0.);
 }
 
 void QtGnuplotEnhanced::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
@@ -108,12 +107,6 @@ QRectF QtGnuplotEnhancedFragment::boundingRect() const
 	return metrics.boundingRect(m_text);
 }
 
-qreal QtGnuplotEnhancedFragment::width() const
-{
-	QFontMetricsF metrics(m_font);
-	return metrics.width(m_text);
-}
-
 void QtGnuplotEnhancedFragment::paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget)
 {
 	Q_UNUSED(option);
@@ -161,11 +154,10 @@ void QtGnuplotPoint::paint(QPainter* painter, const QStyleOptionGraphicsItem* op
 
 void QtGnuplotPoint::drawPoint(QPainter* painter, const QPointF& origin, double size, int style)
 {
-	if (style == -1) // dot
-	{
-		painter->drawPoint(origin);
+	// painter->drawPoint(origin);
+
+	if (style == -1)
 		return;
-	}
 
 	if ((style == 0) || (style == 2)) // plus or star
 	{
