@@ -62,13 +62,12 @@ extern "C" {
 	#include "alloc.h"     // for gp_alloc
 	#include "parse.h"     // for real_expression
 	#include "axis.h"
-#ifdef _WIN32
-	#include "win/winmain.h"  // for WinMessageLoop, ConsoleReadCh
+#ifdef WIN32
+	#include "win/winmain.h"  // for WinMessageLoop, ConsoleReadCh, stdin_pipe_reader
 	#include "win/wgnuplib.h" // for TextStartEditing, TextStopEditing
 	#include "win/wtext.h"    // for kbhit, getchar
 	#include <io.h>           // for isatty
 #endif
-	#include "readline.h"	// for wrap_readline_signal_handler() to catch ^Z
 	#include <signal.h>
 }
 
@@ -106,9 +105,6 @@ struct QtGnuplotState {
     QLocalSocket socket;
     QByteArray   outBuffer;
     QDataStream  out;
-#ifdef _WIN32
-    DWORD        pid;
-#endif
 
     bool       enhancedSymbol;
     QString    enhancedFontName;
@@ -168,9 +164,6 @@ static int  qt_optionWidth    = 640;
 static int  qt_optionHeight   = 480;
 static int  qt_optionFontSize = 9;
 static double qt_optionDashLength = 1.0;
-static double qt_optionLineWidth = 1.0;
-
-static int qt_optionctrlq = -1;	// tristate -1 = not set 0 = false 1 = true 
 
 /* Encapsulates all Qt options that have a constructor and destructor. */
 struct QtOption {
@@ -181,62 +174,43 @@ struct QtOption {
     QString FontName;
     QString Title;
     QString Widget;
-    QPoint  position;
+	QPoint  position;
 };
 static QtOption* qt_option = NULL;
 
 static void ensureOptionsCreated()
 {
 	if (!qt_option)
-		qt_option = new QtOption();
+	    qt_option = new QtOption();
 }
 
 static bool qt_setPosition = false;
 static bool qt_setSize   = true;
 static int  qt_setWidth  = qt_optionWidth;
 static int  qt_setHeight = qt_optionHeight;
-static bool qt_is_3Dplot = false;
-
-static double qt_max_pos_base = 0.0;
-static double qt_max_neg_base = 0.0;
 
 /* ------------------------------------------------------
  * Helpers
  * ------------------------------------------------------*/
-// Adjust for the mismatch of real coords 0->ymax and pixel coords [0:ymax-1]
-#ifndef QT_YBASE
-#define QT_YBASE 1
-#endif
 
 // Convert gnuplot coordinates into floating point term coordinates
 QPointF qt_termCoordF(int x, int y)
 {
-	return QPointF(double(x)/qt_oversamplingF,
-			double(int(term->ymax) - QT_YBASE - y)/qt_oversamplingF);
+	return QPointF(double(x)/qt_oversamplingF, double(int(term->ymax) - y)/qt_oversamplingF);
 }
 
 // The same, but with coordinates clipped to the nearest pixel
 QPoint qt_termCoord(int x, int y)
 {
-	return QPoint(qRound(double(x)/qt_oversamplingF),
-			qRound(double(term->ymax - QT_YBASE - y)/qt_oversamplingF));
+	return QPoint(qRound(double(x)/qt_oversamplingF), qRound(double(term->ymax - y)/qt_oversamplingF));
 }
 
 // Inverse of the previous function
 QPoint qt_gnuplotCoord(int x, int y)
 {
-	return QPoint(x*qt_oversampling,
-			int(term->ymax) - QT_YBASE - y*qt_oversampling);
+	return QPoint(x*qt_oversampling, int(term->ymax) - y*qt_oversampling);
 }
 
-#ifndef GNUPLOT_QT
-# ifdef _WIN32
-#  define GNUPLOT_QT "gnuplot_qt.exe"
-# else
-#  define GNUPLOT_QT "gnuplot_qt"
-# endif
-#endif
-
 // Start the GUI application
 void execGnuplotQt()
 {
@@ -244,29 +218,22 @@ void execGnuplotQt()
 	char* path = getenv("GNUPLOT_DRIVER_DIR");
 	if (path)
 		filename = QString(path);
-	if (filename.isEmpty()) {
-#ifdef _WIN32
+#ifdef WIN32
+	if (filename.isEmpty())
 		filename = QCoreApplication::applicationDirPath();
+	filename += "/gnuplot_qt.exe";
 #else
+	if (filename.isEmpty())
 		filename = QT_DRIVER_DIR;
+	filename += "/gnuplot_qt";
 #endif
-	}
-
-	filename += "/";
-	filename += GNUPLOT_QT;
 
 	qint64 pid;
 	qt->gnuplot_qtStarted = QProcess::startDetached(filename, QStringList(), QString(), &pid);
-	if (qt->gnuplot_qtStarted) {
+	if (qt->gnuplot_qtStarted)
 		qt->localServerName = "qtgnuplot" + QString::number(pid);
-#ifdef _WIN32
-		// save the terminal's PID for later use
-		qt->pid = pid;
-#endif
-	} else {
-		fprintf(stderr, "Could not start gnuplot_qt with path %s\n", filename.toUtf8().data());
-		fprintf(stderr, "Did you set environmental variable GNUPLOT_DRIVER_DIR?\n");
-	}
+	else
+		qDebug() << "Could not start gnuplot_qt with path" << filename;
 }
 
 /*-------------------------------------------------------
@@ -278,7 +245,7 @@ void qt_flushOutBuffer()
 	if (!qt || !qt->socket.isValid())
 		return;
 
-	// Write the block size at the beginning of the block
+	// Write the block size at the beginning of the bock
 	QDataStream sizeStream(&qt->socket);
 	sizeStream << (quint32)(qt->outBuffer.size());
 	// Write the block to the QLocalSocket
@@ -323,14 +290,14 @@ void qt_connectToServer(const QString& server, bool retry = true)
 		// The widget could not be reached: start a gnuplot_qt program which will create a QtGnuplotApplication
 		if (connectToWidget)
 		{
-			fprintf(stderr, "Could not connect to existing qt widget. Starting a new one.\n");
+			qDebug() << "Could not connect to widget" << qt_option->Widget << ". Starting a QtGnuplotApplication";
 			qt_option->Widget = QString();
 			qt_connectToServer(qt->localServerName);
 		}
 		// The gnuplot_qt program could not be reached: try to start a new one
 		else
 		{
-			fprintf(stderr, "Could not connect to existing gnuplot_qt. Starting a new one.\n");
+			qDebug() << "Could not connect to gnuplot_qt" << server << ". Starting a new one";
 			execGnuplotQt();
 			qt_connectToServer(qt->localServerName, false);
 		}
@@ -382,7 +349,7 @@ bool qt_processTermEvent(gp_event_t* event)
 		// This is an answer to a font metric request. We don't send it back to gnuplot
 		if ((event->par1 > 0) && (event->par2 > 0))
 		{
-			fprintf(stderr, "qt_processTermEvent received a GE_fontprops event. This should not have happened\n");
+			qDebug() << "qt_processTermEvent received a GE_fontprops event. This should not have happened";
 			return false;
 		}
 		// This is a resize event
@@ -398,19 +365,6 @@ bool qt_processTermEvent(gp_event_t* event)
 		event->my = p.y();
 	}
 
-#ifdef _WIN32
-	if (event->type == GE_raise)
-	{
-# ifndef WGP_CONSOLE
-		SetForegroundWindow(textwin.hWndParent);
-# else
-		SetForegroundWindow(GetConsoleWindow());
-# endif
-		WinRaiseConsole();
-		return true;
-	}
-#endif
-
 	// Send the event to gnuplot core
 	do_event(event);
 	// Process pause_for_mouse
@@ -429,11 +383,6 @@ bool qt_processTermEvent(gp_event_t* event)
 		paused_for_mouse = 0;
 		return true;
 	}
-	if (event->type == GE_reset)
-	{
-		paused_for_mouse = 0;
-		return true;
-	}
 
 	return false;
 }
@@ -458,7 +407,7 @@ void qt_init()
 	// The creation of a QApplication mangled our locale settings
 #ifdef HAVE_LOCALE_H
 	setlocale(LC_NUMERIC, "C");
-	setlocale(LC_TIME, time_locale);
+	setlocale(LC_TIME, current_locale);
 #endif
 
 	qt->out.setVersion(QDataStream::Qt_4_4);
@@ -496,16 +445,13 @@ void qt_sendFont()
 			if (qt->socket.bytesAvailable() < (int)sizeof(gp_event_t)) {
 				fprintf(stderr, (waitcount++ % 10 > 0) ? "  ."
 					: "\nWarning: slow font initialization");
-#ifndef Q_OS_MAC
+#ifdef Q_OS_MAC
 				// OSX can be slow (>30 seconds?!) in determining font metrics
-				// Always give it more time rather than failing after 1 second
-				// Everyone else can use --slow on the command line
-				if (slow_font_startup)
+				// Give it more time rather than failing after 1 second 
+				// Possibly this is only relevant to Qt5
+				GP_SLEEP(0.5);
+				continue;
 #endif
-				{
-				    GP_SLEEP(0.5);
-				    continue;
-				}
 				return;
 			}
 			while (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t))
@@ -558,20 +504,9 @@ void qt_graphics()
 	// Initialize window
 	qt->out << GESetCurrentWindow << qt_optionWindowId;
 	qt->out << GEInitWindow;
-#ifdef _WIN32
-	// Let the terminal window know our PID
-	qt->out << GEPID << quint32(GetCurrentProcessId());
-#endif
 	qt->out << GEActivate;
 	qt->out << GETitle << qt_option->Title;
-
-	// We used to always send true or false; now it's a tristate
-	// only sent if there was an explicit set/unset in the "set term" command
-	if (qt_optionctrlq >= 0) {
-		qt->out << GESetCtrl << qt_optionCtrl;
-		qt_optionctrlq = -1;
-	}
-
+	qt->out << GESetCtrl << qt_optionCtrl;
 	qt->out << GESetWidgetSize << QSize(term->xmax, term->ymax)/qt_oversampling;
 	// Initialize the scene
 	qt->out << GESetSceneSize << QSize(term->xmax, term->ymax)/qt_oversampling;
@@ -580,8 +515,6 @@ void qt_graphics()
 	qt_sendFont();
 	term->v_tic = (unsigned int) (term->v_char/2.5);
 	term->h_tic = (unsigned int) (term->v_char/2.5);
-	// Tell user what scale factor relates term coordinates to pixels
-	term->tscale = qt_oversamplingF;
 
 	if (qt_setPosition)
 	{
@@ -594,13 +527,7 @@ void qt_graphics()
 void qt_text()
 {
 	if (qt_optionRaise)
-	{
-#ifdef _WIN32
-		// The qt graph window is in another process and we must explicitly allow it to "raise".
-		AllowSetForegroundWindow(qt->pid);
-#endif
 		qt->out << GERaise;
-	}
 	qt->out << GEDone;
 	qt_flushOutBuffer();
 }
@@ -610,7 +537,7 @@ void qt_text_wrapper()
 	if (!qt)
 		return;
 
-	// Remember scale so that we can update the status bar while the plot is inactive
+	// Remember scale to update the status bar while the plot is inactive
 	qt->out << GEScale;
 
 	const int axis_order[4] = {FIRST_X_AXIS, FIRST_Y_AXIS, SECOND_X_AXIS, SECOND_Y_AXIS};
@@ -630,15 +557,13 @@ void qt_text_wrapper()
 		qt->out << lower/qt_oversamplingF << scale/qt_oversamplingF;
 		qt->out << (axis_array[axis_order[i]].log ? axis_array[axis_order[i]].log_base : 0.);
 	}
-	// Flag whether this was a 3D plot (not mousable in 'persist' mode)
-	qt->out << qt_is_3Dplot;
-	qt_is_3Dplot = false;
 
 	qt_text();
 }
 
 void qt_reset()
 {
+	/// @todo
 }
 
 void qt_move(unsigned int x, unsigned int y)
@@ -656,7 +581,7 @@ void qt_enhanced_flush()
 	qt->out << GEEnhancedFlush << qt->enhancedFontName << qt->enhancedFontSize
 		<< (int)qt->enhancedFontStyle << (int)qt->enhancedFontWeight
 		<< qt->enhancedBase << qt->enhancedWidthFlag << qt->enhancedShowFlag
-		<< qt->enhancedOverprint
+		<< qt->enhancedOverprint 
 		<< qt->codec->toUnicode(qt->enhancedText);
 	qt->enhancedText.clear();
 }
@@ -677,12 +602,6 @@ void qt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN wid
 	qt->enhancedShowFlag  = showflag;
 	qt->enhancedOverprint = overprint;
 
-	// Baseline correction.  Surely Qt itself provides this somehow?
-	if (qt_max_pos_base < base)
-		qt_max_pos_base = base;
-	if (qt_max_neg_base > base)
-		qt_max_neg_base = base;
-
 	// strip Bold or Italic property out of font name
 	QString tempname = fontname;
 	if (tempname.contains(":italic", Qt::CaseInsensitive))
@@ -696,7 +615,7 @@ void qt_enhanced_open(char* fontname, double fontsize, double base, TBOOLEAN wid
 	int sep = tempname.indexOf(":");
 	if (sep >= 0)
 		tempname.truncate(sep);
-
+	
 	// Blank font name means keep using the previous font
 	if (!tempname.isEmpty())
 		qt->enhancedFontName = tempname;
@@ -731,9 +650,6 @@ void qt_put_text(unsigned int x, unsigned int y, const char* string)
 	enhanced_fontscale = 1.0;
 	strncpy(enhanced_escape_format, "%c", sizeof(enhanced_escape_format));
 
-	// Baseline correction
-	qt_max_pos_base = qt_max_neg_base = 0.0;
-
 	// Set the recursion going. We say to keep going until a closing brace, but
 	// we don't really expect to find one.  If the return value is not the nul-
 	// terminator of the string, that can only mean that we did find an unmatched
@@ -749,27 +665,26 @@ void qt_put_text(unsigned int x, unsigned int y, const char* string)
 		// else carry on and process the rest of the string
 	}
 
-	// Baseline correction
-	y += qt_max_pos_base * 5;
-	y += qt_max_neg_base * 5;
-
 	qt->out << GEEnhancedFinish << qt_termCoord(x, y);
 }
 
 void qt_linetype(int lt)
 {
+	if (lt <= LT_NODRAW)
+		lt = LT_NODRAW; // background color
+
 	/* Version 5: dash pattern will be set later by term->dashtype */
 	if (lt == LT_AXIS)
 		qt->out << GEPenStyle << Qt::DotLine;
 	else if (lt == LT_NODRAW)
 		qt->out << GEPenStyle << Qt::NoPen;
-	else
+	else 
 		qt->out << GEPenStyle << Qt::SolidLine;
 
-	if (lt == LT_BACKGROUND) {
+	if ((lt-1) == LT_BACKGROUND) {
 		/* FIXME: Add parameter to this API to set the background color from the gnuplot end */
 		qt->out << GEBackgroundColor;
-	} else if (lt > LT_NODRAW)
+	} else
 		qt->out << GEPenColor << qt_colorList[lt % 9 + 3];
 }
 
@@ -839,7 +754,7 @@ int qt_set_font(const char* font)
 
 	/* Optimize by leaving early if there is no change */
 	if (qt->currentFontSize == qt_previousFontSize
-	&&  qt->currentFontName == qt_previousFontName) {
+	&&  qt->currentFontName == qt->currentFontName) {
 		return 1;
 	}
 
@@ -873,7 +788,7 @@ void qt_pointsize(double ptsize)
 
 void qt_linewidth(double lw)
 {
-	qt->out << GELineWidth << lw * qt_optionLineWidth;
+	qt->out << GELineWidth << lw;
 }
 
 int qt_text_angle(int angle)
@@ -884,14 +799,8 @@ int qt_text_angle(int angle)
 
 void qt_fillbox(int style, unsigned int x, unsigned int y, unsigned int width, unsigned int height)
 {
-	// If y >= ymax then no box is drawn. So we clip.
-	unsigned int ytop = y + height;
-	if (ytop >= term->ymax) {
-	    ytop = term->ymax-1;
-	    height = ytop - y;
-	}
 	qt->out << GEBrushStyle << style;
-	qt->out << GEFillBox << QRect(qt_termCoord(x, ytop), QSize(width, height)/qt_oversampling);
+	qt->out << GEFillBox << QRect(qt_termCoord(x, y + height), QSize(width, height)/qt_oversampling);
 }
 
 int qt_make_palette(t_sm_palette* palette)
@@ -951,8 +860,8 @@ void qt_image(unsigned int M, unsigned int N, coordval* image, gpiPoint* corner,
 // box, with \r separating text above and below the point.
 void qt_put_tmptext(int n, const char str[])
 {
-	if (!qt)
-		return;
+    if (!qt)
+        return;
 
 	if (n == 0)
 		qt->out << GEStatusText << QString(str);
@@ -966,8 +875,8 @@ void qt_put_tmptext(int n, const char str[])
 
 void qt_set_cursor(int c, int x, int y)
 {
-	if (!qt)
-		return;
+    if (!qt)
+        return;
 
 	// Cancel zoombox when Echap is pressed
 	if (c == 0)
@@ -1014,18 +923,14 @@ void qt_set_clipboard(const char s[])
 int qt_waitforinput(int options)
 {
 #ifdef USE_MOUSE
-#ifndef _WIN32
+#ifndef WIN32
 	fd_set read_fds;
 	struct timeval one_msec;
 	int stdin_fd  = fileno(stdin);
 	int socket_fd = qt ? qt->socket.socketDescriptor() : -1;
 
-	if (!qt || (socket_fd < 0) || (qt->socket.state() != QLocalSocket::ConnectedState)) {
-		if (options == TERM_ONLY_CHECK_MOUSING)
-			return '\0';
-		else
-			return getchar();
-	}
+	if (!qt || (socket_fd < 0) || (qt->socket.state() != QLocalSocket::ConnectedState))
+		return (options == TERM_ONLY_CHECK_MOUSING) ? '\0' : getchar();
 
 	// Gnuplot event loop
 	do
@@ -1048,16 +953,8 @@ int qt_waitforinput(int options)
 		}
 
 		// Wait for input
-		int n_changed_fds = select(socket_fd+1, &read_fds,
-						NULL,	// not watching for write-only
-						NULL,	// not watching for exceptions
-						timeout );
-
-		if (n_changed_fds < 0) {
-			if (errno == EINTR) {
-				wrap_readline_signal_handler();
-				continue;
-			}
+		if (select(socket_fd+1, &read_fds, NULL, NULL, timeout) < 0)
+		{
 			// Display the error message except when Ctrl + C is pressed
 			if (errno != 4)
 				fprintf(stderr, "Qt terminal communication error: select() error %i %s\n", errno, strerror(errno));
@@ -1069,7 +966,7 @@ int qt_waitforinput(int options)
 		{
 			if (!(qt->socket.waitForReadyRead(-1))) {
 				// Must be a socket error; we need to restart qt_gnuplot
-				fprintf(stderr, "Error: plot window (gnuplot_qt) not responding - will restart\n");
+				qDebug() << "Error: gnuplot_qt socket not responding";
 				qt->gnuplot_qtStarted = false;
 				return '\0';
 			}
@@ -1079,7 +976,7 @@ int qt_waitforinput(int options)
 			gp_event_t tempEvent;
 			tempEvent.type = -1;
 			if (qt->socket.bytesAvailable() < (int)sizeof(gp_event_t)) {
-				fprintf(stderr, "Error: short read from gnuplot_qt socket\n");
+				qDebug() << "Error: short read from gnuplot_qt socket";
 				return '\0';
 			}
 			while (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t))
@@ -1110,13 +1007,8 @@ int qt_waitforinput(int options)
 			return '\0';
 		}
 	} while (paused_for_mouse || !FD_ISSET(stdin_fd, &read_fds));
-
-	if (options == TERM_ONLY_CHECK_MOUSING)
-		return '\0';
-
-	return getchar();
-
-#else // _WIN32, Windows console and wgnuplot
+		return getchar();
+#else // Windows console and wgnuplot
 #ifdef WGP_CONSOLE
 	int fd = fileno(stdin);
 #endif
@@ -1137,7 +1029,7 @@ int qt_waitforinput(int options)
 	if (options != TERM_ONLY_CHECK_MOUSING) { // NOTE: change this if used also for the caca terminal
 #ifdef WGP_CONSOLE
 		if (!isatty(fd))
-			h[0] = (HANDLE)_get_osfhandle(fd);
+			h[0] = CreateThread(NULL, 0, stdin_pipe_reader, NULL, 0, NULL);
 		else
 #endif
 			h[0] = GetStdHandle(STD_INPUT_HANDLE);
@@ -1163,7 +1055,7 @@ int qt_waitforinput(int options)
 
 	do {
 		int waitResult = -1;
-
+		
 #ifndef WGP_CONSOLE
 		// Process pending key events of the text console
 		if (kbhit() && (options != TERM_ONLY_CHECK_MOUSING))
@@ -1174,21 +1066,21 @@ int qt_waitforinput(int options)
 		if ((idx_socket != -1) && // (qt != NULL)) &&
 			(qt->socket.waitForReadyRead(0)) && (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t)))
 			waitResult = idx_socket; // data already available
-
-		// Wait for a new event
+		
+		// Wait for a new event 
 		if ((waitResult == -1) && (options != TERM_ONLY_CHECK_MOUSING))
 			waitResult = MsgWaitForMultipleObjects(idx, h, FALSE, INFINITE, QS_ALLINPUT);  // wait for new data
-
+		
 		if ((waitResult == idx_stdin) && (idx_stdin != -1)) { // console windows or caca terminal (TBD)
 #ifdef WGP_CONSOLE
 			if (!isatty(fd)) {
-				unsigned char ch;
-				if (fread(&ch, 1, 1, stdin) == 1)
-					c = ch;
-				else
-					c = EOF;
+				DWORD dw;
+
+				GetExitCodeThread(h[0], &dw);
+				CloseHandle(h[0]);
+				c = dw;
 				quitLoop = true;
-			} else
+			} else 
 #endif
 			{
 				c = ConsoleReadCh();
@@ -1203,7 +1095,6 @@ int qt_waitforinput(int options)
 			// are received, only transmit the last one.
 			gp_event_t tempEvent;
 			tempEvent.type = -1;
-			int size = qt->socket.bytesAvailable();
 			while (qt->socket.bytesAvailable() >= (int)sizeof(gp_event_t)) {
 				struct gp_event_t event;
 				qt->socket.read((char*) &event, sizeof(gp_event_t));
@@ -1222,10 +1113,6 @@ int qt_waitforinput(int options)
 					}
 				}
 			}
-			// If the native pipe handle signalled new data, but the QtLocalSocket
-			// object has no data available, release the CPU for a little while.
-			if (size == 0)
-				Sleep(100);
 			// Replay move event
 			if (tempEvent.type == GE_motion)
 				qt_processTermEvent(&tempEvent);
@@ -1256,18 +1143,17 @@ int qt_waitforinput(int options)
 		}
 	} while (!quitLoop);
 
+
 #ifndef WGP_CONSOLE
 	if (options != TERM_ONLY_CHECK_MOUSING)
 		TextStopEditing(&textwin);
-
+	
 	// This happens if neither the qt queue is alive, nor there is a console window.
 	if ((options != TERM_ONLY_CHECK_MOUSING) && !waitOK)
 		return getchar();
-#else
-	(void) waitOK;
 #endif
 	return c;
-#endif // _WIN32
+#endif // WIN32
 #else
 	return getchar();
 #endif // USE_MOUSE
@@ -1290,9 +1176,9 @@ void qt_atexit()
 	else
 		qt->out << GEExit;
 	qt_flushOutBuffer();
-
-	delete qt;
-	qt = NULL;
+        
+        delete qt;
+        qt = NULL;
 
 	delete qt_option;
 	qt_option = NULL;
@@ -1320,7 +1206,6 @@ enum QT_id {
 	QT_DASH,
 	QT_DASHLENGTH,
 	QT_SOLID,
-	QT_LINEWIDTH,
 	QT_OTHER
 };
 
@@ -1339,12 +1224,10 @@ static struct gen_table qt_opts[] = {
 	{"noct$rlq",    QT_NOCTRL},
 	{"ti$tle",      QT_TITLE},
 	{"cl$ose",      QT_CLOSE},
-	{"dash$ed",     QT_DASH},
-	{"dashl$ength", QT_DASHLENGTH},
-	{"dl",          QT_DASHLENGTH},
-	{"solid",       QT_SOLID},
-	{"line$width",  QT_LINEWIDTH},
-	{"lw",          QT_LINEWIDTH},
+	{"dash$ed",	QT_DASH},
+	{"dashl$ength",	QT_DASHLENGTH},
+	{"dl",		QT_DASHLENGTH},
+	{"solid",	QT_SOLID},
 	{NULL,          QT_OTHER}
 };
 
@@ -1367,10 +1250,8 @@ void qt_options()
 	bool set_widget = false;
 	bool set_dash = false;
 	bool set_dashlength = false;
-	bool set_linewidth = false;
-	int previous_WindowId = qt_optionWindowId;
 
-#ifndef _WIN32
+#ifndef WIN32
 	if (term_interlock != NULL && term_interlock != (void *)qt_init) {
 		term = NULL;
 		int_error(NO_CARET, "The qt terminal cannot be used in a wxt session");
@@ -1485,10 +1366,6 @@ void qt_options()
 			// qt_optionDash = false;
 			c_token++;
 			break;
-		case QT_LINEWIDTH:
-			SETCHECKDUP(set_linewidth);
-			qt_optionLineWidth = real_expression();
-			break;
 		case QT_OTHER:
 		default:
 			qt_optionWindowId = int_expression();
@@ -1502,14 +1379,6 @@ void qt_options()
 			int_error(c_token-1, "Duplicated or contradicting arguments in qt term options.");
 	}
 
-	// We want this to happen immediately, hence the flush command.
-	// We don't want it to change the _current_ window, just close an old one.
-	if (set_close && qt) {
-		qt->out << GECloseWindow << qt_optionWindowId;
-		qt_flushOutBuffer();
-		qt_optionWindowId = previous_WindowId;
-	}
-
 	// Save options back into options string in normalized format
 	QString termOptions = QString::number(qt_optionWindowId);
 
@@ -1519,6 +1388,8 @@ void qt_options()
 	if (set_title)
 	{
 		termOptions += " title \"" + qt_option->Title + '"';
+		if (qt)
+			qt->out << GETitle << qt_option->Title;
 	}
 
 	if (set_size)
@@ -1537,21 +1408,15 @@ void qt_options()
 		qt_setPosition = true;
 	}
 
-	termOptions += " font \"" + fontSettings + '"';
-
 	if (set_enhanced) termOptions += qt_optionEnhanced ? " enhanced" : " noenhanced";
-	if (set_linewidth) termOptions += " linewidth " + QString::number(qt_optionLineWidth);
+	                  termOptions += " font \"" + fontSettings + '"';
 	if (set_dashlength) termOptions += " dashlength " + QString::number(qt_optionDashLength);
 	if (set_widget)   termOptions += " widget \"" + qt_option->Widget + '"';
 	if (set_persist)  termOptions += qt_optionPersist ? " persist" : " nopersist";
 	if (set_raise)    termOptions += qt_optionRaise ? " raise" : " noraise";
 	if (set_ctrl)     termOptions += qt_optionCtrl ? " ctrl" : " noctrl";
 
-	/// Only send the ctrlQ option if it is explicitly given in "set term qt {no}ctrlq"
-	if (set_ctrl)
-	    qt_optionctrlq = qt_optionCtrl;
-	else
-	    qt_optionctrlq = -1;
+	if (set_close && qt) qt->out << GECloseWindow << qt_optionWindowId;
 
 	/// @bug change Utf8 to local encoding
 	strncpy(term_options, termOptions.toUtf8().data(), MAX_LINE_LEN);
@@ -1559,12 +1424,12 @@ void qt_options()
 
 void qt_layer( t_termlayer syncpoint )
 {
-	static int current_plotno = 0;
+    static int current_plotno = 0;
 	if (!qt)
 		return;
 
-	/* We must ignore all syncpoints that we don't recognize */
-	switch (syncpoint) {
+    /* We must ignore all syncpoints that we don't recognize */
+    switch (syncpoint) {
 	case TERM_LAYER_BEFORE_PLOT:
 		current_plotno++;
 		qt->out << GEPlotNumber << current_plotno; break;
@@ -1586,11 +1451,9 @@ void qt_layer( t_termlayer syncpoint )
 		qt->out << GELayer << QTLAYER_END_KEYSAMPLE; break;
 	case TERM_LAYER_BEFORE_ZOOM:
 		qt->out << GELayer << QTLAYER_BEFORE_ZOOM; break;
-	case TERM_LAYER_3DPLOT:
-		qt_is_3Dplot = true; break;
-	default:
+    	default:
 		break;
-	}
+    }
 }
 
 void qt_hypertext( int type, const char *text )
@@ -1599,16 +1462,12 @@ void qt_hypertext( int type, const char *text )
 		qt->out << GEHypertext << qt->codec->toUnicode(text);
 }
 
+#ifdef EAM_BOXED_TEXT
 void qt_boxed_text(unsigned int x, unsigned int y, int option)
 {
-	if (option == TEXTBOX_MARGINS) {
-	    // slightly largin y margin to allow for descenders
-	    double xmargin = (double)x/(100*qt_oversamplingF);
-	    double ymargin = (double)y/(100*qt_oversamplingF) + 0.75/qt_oversamplingF;
-	    qt->out << GETextBox << QPointF(xmargin, ymargin) << option;
-	} else
-	    qt->out << GETextBox << qt_termCoordF(x, y) << option;
+	qt->out << GETextBox << qt_termCoordF(x, y) << option;
 }
+#endif
 
 void qt_modify_plots(unsigned int ops, int plotno)
 {
