@@ -1,3 +1,7 @@
+/*
+ * $Id: stdfn.h,v 1.49 2014/05/01 19:50:15 broeker Exp $
+ */
+
 /* GNUPLOT - stdfn.h */
 
 /*[
@@ -60,7 +64,7 @@
 # endif
 #else
 # ifndef HAVE_MEMCPY
-char * memcpy(char *, char *, size_t);
+char * memcpy __PROTO((char *, char *, size_t));
 # endif
 #endif /* HAVE_BCOPY */
 
@@ -71,7 +75,7 @@ char * memcpy(char *, char *, size_t);
 # ifdef HAVE_INDEX
 #  define strchr index
 # else
-char *strchr(const char *, int);
+char *strchr __PROTO((const char *, int));
 # endif
 # ifdef strrchr
 #  undef strrchr
@@ -81,12 +85,12 @@ char *strchr(const char *, int);
 # endif
 #endif
 #ifndef HAVE_STRCSPN
-size_t gp_strcspn(const char *, const char *);
+size_t gp_strcspn __PROTO((const char *, const char *));
 # define strcspn gp_strcspn
 #endif
 
 #ifndef HAVE_STRSTR
-char *strstr(const char *, const char *);
+char *strstr __PROTO((const char *, const char *));
 #endif
 
 #ifndef HAVE_STDLIB_H
@@ -146,7 +150,7 @@ double strtod();
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #else
-# ifdef HAVE_LIBC_H
+# ifdef HAVE_LIBC_H /* NeXT uses libc instead of unistd */
 #  include <libc.h>
 # endif
 #endif /* HAVE_UNISTD_H */
@@ -158,7 +162,7 @@ double strtod();
 extern int errno;
 #endif
 #ifndef HAVE_STRERROR
-char *strerror(int);
+char *strerror __PROTO((int));
 extern int sys_nerr;
 extern char *sys_errlist[];
 #endif
@@ -262,8 +266,8 @@ extern char *sys_errlist[];
 #endif
 
 #if defined(PIPES) && defined(VMS)
-FILE *popen(char *, char *);
-int pclose(FILE *);
+FILE *popen __PROTO((char *, char *));
+int pclose __PROTO((FILE *));
 #endif
 
 #ifdef HAVE_FLOAT_H
@@ -306,7 +310,7 @@ int pclose(FILE *);
 # ifdef HAVE_STRICMP
 #  define strcasecmp stricmp
 # else
-int gp_stricmp(const char *, const char *);
+int gp_stricmp __PROTO((const char *, const char *));
 #  define strcasecmp gp_stricmp
 # endif
 #endif
@@ -315,29 +319,26 @@ int gp_stricmp(const char *, const char *);
 # ifdef HAVE_STRNICMP
 #  define strncasecmp strnicmp
 # else
-int gp_strnicmp(const char *, const char *, size_t);
+int gp_strnicmp __PROTO((const char *, const char *, size_t));
 #  define strncasecmp gp_strnicmp
 # endif
 #endif
 
 #ifndef HAVE_STRNDUP
-char *strndup(const char * str, size_t n);
+char *strndup __PROTO((const char * str, size_t n));
 #endif
 
 #ifndef HAVE_STRNLEN
-size_t strnlen(const char *str, size_t n);
+size_t strnlen __PROTO((const char *str, size_t n));
 #endif
 
-#if defined(_MSC_VER) && (_MSC_VER < 1900)
+#if defined(_MSC_VER)
 int ms_vsnprintf(char *str, size_t size, const char *format, va_list ap);
 int ms_snprintf(char *str, size_t size, const char * format, ...);
 #endif
 
 #ifndef GP_GETCWD
-# if defined(_WIN32)
-#  define GP_GETCWD(path,len) gp_getcwd (path, len)
-char * gp_getcwd(char *path, size_t len);
-# elif defined(HAVE_GETCWD)
+# if defined(HAVE_GETCWD)
 #   if defined(__EMX__)
 #     define GP_GETCWD(path,len) _getcwd2 (path, len)
 #   else
@@ -348,8 +349,12 @@ char * gp_getcwd(char *path, size_t len);
 # endif
 #endif
 
+#ifdef WIN32
+# include <windows.h>
+#endif
+
 /* sleep delay time, where delay is a double value */
-#if defined(HAVE_USLEEP) && !defined(_WIN32)
+#if defined(HAVE_USLEEP)
 #  define GP_SLEEP(delay) usleep((unsigned int) ((delay)*1e6))
 #  ifndef HAVE_SLEEP
 #    define HAVE_SLEEP
@@ -359,7 +364,7 @@ char * gp_getcwd(char *path, size_t len);
 #  ifndef HAVE_SLEEP
 #    define HAVE_SLEEP
 #  endif
-#elif defined(_WIN32)
+#elif defined(WIN32)
 #  define GP_SLEEP(delay) win_sleep((DWORD) 1000*delay)
 #  ifndef HAVE_SLEEP
 #    define HAVE_SLEEP
@@ -371,12 +376,12 @@ char * gp_getcwd(char *path, size_t len);
 #endif
 
 /* Gnuplot replacement for atexit(3) */
-void gp_atexit(void (*function)(void));
+void gp_atexit __PROTO((void (*function)(void)));
 
 /* Gnuplot replacement for exit(3). Calls the functions registered using
  * gp_atexit(). Always use this function instead of exit(3)!
  */
-void gp_exit(int status);
+void gp_exit __PROTO((int status));
 
 /* Calls the cleanup functions registered using gp_atexit().
  * Normally gnuplot should be exited using gp_exit(). In some cases, this is not
@@ -384,11 +389,11 @@ void gp_exit(int status);
  * confused because they expect a return statement at the very end. In that
  * case, gp_exit_cleanup() should be called before the return statement.
  */
-void gp_exit_cleanup(void);
+void gp_exit_cleanup __PROTO((void));
 
-char * gp_basename(char *path);
+char * gp_basename __PROTO((char *path));
 
-#ifdef _WIN32
+#if !defined(HAVE_DIRENT_H) && defined(WIN32) && (!defined(__WATCOMC__))
 /*
 
     Declaration of POSIX directory browsing functions and types for Win32.
@@ -404,39 +409,28 @@ char * gp_basename(char *path);
     documentation for any purpose is hereby granted without fee, provided
     that this copyright and permissions notice appear in all copies and
     derivatives.
-
+    
     This software is supplied "as is" without express or implied warranty.
 
     But that said, if there are any problems please get in touch.
 
 */
-typedef struct GPDIR GPDIR;
+typedef struct DIR DIR;
 
-struct gp_dirent
+struct dirent
 {
     char *d_name;
 };
 
-GPDIR * gp_opendir(const char *);
-int gp_closedir(GPDIR *);
-struct gp_dirent *gp_readdir(GPDIR *);
-void gp_rewinddir(GPDIR *);
-#define opendir(p) gp_opendir(p)
-#define closedir(d) gp_closedir(d)
-#define readdir(d) gp_readdir(d)
-#define rewinddir(d) gp_rewinddir(d)
-#define dirent gp_dirent
-#define DIR GPDIR
+DIR           *opendir __PROTO((const char *));
+int           closedir __PROTO((DIR *));
+struct dirent *readdir __PROTO((DIR *));
+void          rewinddir __PROTO((DIR *));
 #elif defined(HAVE_DIRENT_H)
 # include <sys/types.h>
 # include <dirent.h>
-#elif defined(__WATCOMC__)
-# include <direct.h>
-#endif /* !HAVE_DIRENT_H && _WIN32 */
+#endif /* !HAVE_DIRENT_H && WIN32 */
 
-#if defined(HAVE_DIRENT_H) || defined(_WIN32) || defined(__WATCOMC__)
-# define HAVE_DIRENT
-#endif
 
 /* Misc. defines */
 
@@ -455,14 +449,41 @@ void gp_rewinddir(GPDIR *);
 # define DEBUG_WHERE do { fprintf(stderr,"%s:%d ",__FILE__,__LINE__); } while (0)
 # define FPRINTF(a) do { DEBUG_WHERE; fprintf a; } while (0)
 #else
-# define DEBUG_WHERE     do { /* nought */ } while(0)
-# define FPRINTF(a)      do { /* nought */ } while(0)
+# define DEBUG_WHERE     /* nought */
+# define FPRINTF(a)      /* nought */
 #endif /* DEBUG */
 
 #include "syscfg.h"
 
 #define INT_STR_LEN (3*sizeof(int))
 
+
+/* HBB 20010223: moved this whole block from syscfg.h to here. It
+ * needs both "syscfg.h" and <float.h> to have been #include'd before
+ * this, since it relies on stuff like DBL_MAX */
+
+/* There is a bug in the NEXT OS. This is a workaround. Lookout for
+ * an OS correction to cancel the following dinosaur
+ *
+ * Hm, at least with my setup (compiler version 3.1, system 3.3p1),
+ * DBL_MAX is defined correctly and HUGE and HUGE_VAL are both defined
+ * as 1e999. I have no idea to which OS version the bugfix below
+ * applies, at least wrt. HUGE, it is inconsistent with the current
+ * version. Since we are using DBL_MAX anyway, most of this isn't
+ * really needed anymore.
+ */
+
+#if defined ( NEXT ) && NX_CURRENT_COMPILER_RELEASE<310
+# if defined ( DBL_MAX)
+#  undef DBL_MAX
+# endif
+# define DBL_MAX 1.7976931348623157e+308
+# undef HUGE
+# define HUGE    DBL_MAX
+# undef HUGE_VAL
+# define HUGE_VAL DBL_MAX
+#endif /* NEXT && NX_CURRENT_COMPILER_RELEASE<310 */
+
 /*
  * Note about VERYLARGE:  This is the upper bound double (or float, if PC)
  * numbers. This flag indicates very large numbers. It doesn't have to
@@ -516,7 +537,7 @@ void gp_rewinddir(GPDIR *);
 #endif
 /* HBB 2012-03-18: clang brings its own <limits.h>, which lacks PATH_MAX,
  * and on top of that, Cygwin's MAXPATHLEN is defined by reference to PATH_MAX.
- * So even though clang sees a #defined MAXPATHLEN, there's still no
+ * So even though clang sees a #defined MAXPATHLEN, there's still no 
  * definition available OA*/
 #ifndef PATH_MAX
 # if !defined(MAXPATHLEN) || (MAXPATHLEN <= 0)
@@ -569,11 +590,6 @@ void gp_rewinddir(GPDIR *);
     } while (0)
 #endif
 
-#ifndef clip_to_01
-#define clip_to_01(val)	\
-    ((val) < 0 ? 0 : (val) > 1 ? 1 : (val))
-#endif
-
 /* both min/max and MIN/MAX are defined by some compilers.
  * we are now on GPMIN / GPMAX
  */
@@ -587,11 +603,11 @@ void gp_rewinddir(GPDIR *);
 
 /* Prototypes from "stdfn.c" */
 
-char *safe_strncpy(char *, const char *, size_t);
+char *safe_strncpy __PROTO((char *, const char *, size_t));
 #ifndef HAVE_SLEEP
-unsigned int sleep(unsigned int);
+unsigned int sleep __PROTO((unsigned int));
 #endif
 
-double not_a_number(void);
+double not_a_number __PROTO((void));
 
 #endif /* STDFN_H */
