@@ -1,3 +1,7 @@
+#ifndef lint
+static char *RCSid() { return RCSid("$Id: time.c,v 1.28.2.2 2015/01/16 22:01:52 sfeam Exp $"); }
+#endif
+
 /* GNUPLOT - time.c */
 
 /*[
@@ -31,10 +35,15 @@
 ]*/
 
 
-/* This module provides a set of routines to read or write times using
- * human-friendly units (hours, days, years, etc).  Unlike similar
- * ansi routines, these ones support sub-second precision and relative
- * times.  E.g.   -1.23 seconds is a valid time.
+/* This module either adds a routine gstrptime() to read a formatted time,
+ * augmenting the standard suite of time routines provided by ansi,
+ * or it completely replaces the whole lot with a new set of routines,
+ * which count time relative to the year 2000. Default is to use the
+ * new routines. Define USE_SYSTEM_TIME to use the system routines, at your
+ * own risk. One problem in particular is that not all systems allow
+ * the time with integer value 0 to be represented symbolically, which
+ * prevents use of relative times.  Also, the system routines do not allow
+ * you to read in fractional seconds.
  */
 
 
@@ -43,7 +52,7 @@
 #include "util.h"
 #include "variable.h"
 
-static char *read_int(char *s, int nr, int *d);
+static char *read_int __PROTO((char *s, int nr, int *d));
 
 static char *
 read_int(char *s, int nr, int *d)
@@ -58,11 +67,18 @@ read_int(char *s, int nr, int *d)
 }
 
 
-static int gdysize(int yr);
+
+#ifndef USE_SYSTEM_TIME
+
+/* a new set of routines to completely replace the ansi ones
+ * Use at your own risk
+ */
+
+static int gdysize __PROTO((int yr));
 
 static int mndday[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 
-static size_t xstrftime(char *buf, int bsz, const char *fmt, struct tm * tm, double usec, double fulltime);
+static size_t xstrftime __PROTO((char *buf, size_t bufsz, const char *fmt, struct tm * tm, double usec, double fulltime));
 
 /* days in year */
 static int
@@ -78,38 +94,21 @@ gdysize(int yr)
 }
 
 
-/* gstrptime() interprets a time_spec format string
- * and fills in a time structure that can be passed to gmtime() to
- * recover number of seconds from the EPOCH date.
- * Return value:
- * DT_TIMEDATE	indicates "normal" format elements corresponding to
- *		a date that is returned in tm, with fractional seconds
- *		returned in usec
- * DT_DMS	indicates relative time format elements were encountered
- *		(tD tH tM tS).  The relative time in seconds is returned
- *		in reltime.
- * DT_BAD	time format could not be interpreted
- *
- * parameters and return values revised for gnuplot version 5.3
- */
+/* new strptime() and gmtime() to allow time to be read as 24 hour,
+ * and spaces in the format string. time is converted to seconds from
+ * year 2000.... */
 
-td_type
-gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
+char *
+gstrptime(char *s, char *fmt, struct tm *tm, double *usec)
 {
     int yday = 0;
     TBOOLEAN sanity_check_date = FALSE;
-    TBOOLEAN reltime_formats = FALSE;
-    TBOOLEAN explicit_pm = FALSE;
-    TBOOLEAN explicit_am = FALSE;
-    TBOOLEAN leading_minus_sign = FALSE;
 
     tm->tm_mday = 1;
     tm->tm_mon = tm->tm_hour = tm->tm_min = tm->tm_sec = 0;
     /* make relative times work (user-defined tic step) */
     tm->tm_year = ZERO_YEAR;
 
-    init_timezone(tm);
-
     /* Fractional seconds will be returned separately, since
      * there is no slot for the fraction in struct tm.
      */
@@ -138,12 +137,6 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	tm->tm_mon = -1;
 	sanity_check_date = TRUE;
     }
-    /* Relative time formats tD tH tM tS cannot be mixed with date formats */
-    if (strstr(fmt,"%t")) {
-	reltime_formats = TRUE;
-	*reltime = 0.0;
-	sanity_check_date = FALSE;
-    }
 
 
     while (*fmt) {
@@ -256,116 +249,21 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 		double ufraction = 0;
 		double when = strtod (s, &s) - SEC_OFFS_SYS;
 		ggmtime(tm, when);
-		if (fraction && fraction < s)
+		if (fraction)
 		    ufraction = atof(fraction);
-		if (ufraction < 1.)	/* Filter out e.g. 123.456e7 */
+		if (ufraction < 1.)		/* Filter out e.g. 123.456e7 */
 		    *usec = ufraction;
-		*reltime = when;	/* not used unless we return DT_DMS ... */
-		if (when < 0)		/* ... which we force for negative times */
-		    reltime_formats = TRUE;
 		break;
 	    }
 
-	case 't':
-	    /* Relative time formats tD tH tM tS */
-	    {
-		double cont = 0;
-
-		/* Special case of negative time with first field 0,
-		 * e.g.  -00:12:34
-		 */
-		if (*reltime == 0) {
-		    while (isspace(*s)) s++;
-		    if (*s == '-')
-			leading_minus_sign = TRUE;
-		}
-
-		fmt++;
-		if (*fmt == 'D') {
-		    cont = 86400. * strtod(s, &s);
-		} else if (*fmt == 'H') {
-		    cont = 3600. * strtod(s, &s);
-		} else if (*fmt == 'M') {
-		    cont = 60. * strtod(s, &s);
-		} else if (*fmt == 'S') {
-		    cont = strtod(s, &s);
-		} else {
-		    return DT_BAD;
-		}
-		if (*reltime < 0)
-		    *reltime -= fabs(cont);
-		else if (*reltime > 0)
-		    *reltime += fabs(cont);
-		else if (leading_minus_sign)
-		    *reltime -= fabs(cont);
-		else
-		    *reltime = cont;
-		/* FIXME:  leading precision field should be accepted but ignored */
-		break;
-	    }
-
-	case 'a':		/* weekday name (ignored) */
-	case 'A':		/* weekday name (ignored) */
-	    while (isalpha(*s))
-		s++;
-	    break;
-	case 'w':		/* one or two digit weekday number (ignored) */
-	case 'W':		/* one or two digit week number (ignored) */
-	    if (isdigit(*s))
-		s++;
-	    if (isdigit(*s))
-		s++;
-	    break;
-
-	case 'p':		/* am or pm */
-	    if (!strncmp(s, "pm", 2) || !strncmp(s, "PM", 2))
-		explicit_pm = TRUE;
-	    if (!strncmp(s, "am", 2) || !strncmp(s, "AM", 2))
-		explicit_am = TRUE;
-	    s += 2;
-	    break;
-
-#ifdef HAVE_STRUCT_TM_TM_GMTOFF
-	case 'z':		/* timezone offset  */
-	    {
-		int neg = (*s == '-') ? -1 : 1;
-		int off_h, off_m;
-		if (*s == '-' || *s == '+')
-		    s++;
-		s = read_int(s, 2, &off_h);
-		if (*s == ':')
-		    s++;
-		s = read_int(s, 2, &off_m);
-		tm->tm_gmtoff = 3600*off_h + 60*off_m;
-		tm->tm_gmtoff *= neg;
-
-	        break;
-	    }
-	case 'Z':		/* timezone name (ignored) */
-	    while (*s && !isspace(*s))
-		s++;
-	    break;
-#endif
-
 	default:
 	    int_warn(DATAFILE, "Bad time format in string");
 	}
 	fmt++;
     }
 
-    /* Relative times are easy.  Just return the value in reltime */
-    if (reltime_formats) {
-	return DT_DMS;
-    }
-
     FPRINTF((stderr, "read date-time : %02d/%02d/%d:%02d:%02d:%02d\n", tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec));
 
-    /* apply AM/PM correction */
-    if ((tm->tm_hour < 12) && explicit_pm)
-	tm->tm_hour += 12;
-    if ((tm->tm_hour == 12) && explicit_am)
-	tm->tm_hour = 0;
-
     /* now sanity check the date/time entered, normalising if necessary
      * read_int cannot read a -ve number, but can read %m=0 then decrement
      * it to -1
@@ -399,7 +297,8 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	if (yday) {
 
 	    if (tm->tm_yday < 0) {
-		return DT_BAD;
+		// int_error(DATAFILE, "Illegal day of year");
+		return (NULL);
 	    }
 
 	    /* we just set month to jan, day to yday, and let the
@@ -411,10 +310,12 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	    tm->tm_mday = tm->tm_yday + 1;
 	}
 	if (tm->tm_mon < 0) {
-	    return DT_BAD;
+	    // int_error(DATAFILE, "illegal month");
+	    return (NULL);
 	}
 	if (tm->tm_mday < 1) {
-	    return DT_BAD;
+	    // int_error(DATAFILE, "illegal day of month");
+	    return (NULL);
 	}
 	if (tm->tm_mon > 11) {
 	    tm->tm_year += tm->tm_mon / 12;
@@ -430,7 +331,7 @@ gstrptime(char *s, char *fmt, struct tm *tm, double *usec, double *reltime)
 	    }
 	}
     }
-    return DT_TIMEDATE;
+    return (s);
 }
 
 size_t
@@ -450,7 +351,7 @@ gstrftime(char *s, size_t bsz, const char *fmt, double l_clock)
 static size_t
 xstrftime(
     char *str,			/* output buffer */
-    int bsz,			/* remaining space available in buffer */
+    size_t bsz,			/* space available */
     const char *fmt,
     struct tm *tm,
     double usec,
@@ -461,9 +362,6 @@ xstrftime(
     char *s = str;
     TBOOLEAN sign_printed = FALSE;
 
-    if (bsz <= 0)
-	return 0;
-
     memset(str, '\0', bsz);
 
     while (*fmt != '\0') {
@@ -633,46 +531,22 @@ xstrftime(
 		int tminute, tsecond;
 
 		    switch (*fmt++) {
-		    case 'D':
-			/* +/- fractional days */
-			if (p > 0) {
-			    incr = snprintf(s, bsz-l-1, "%*.*f", w, p, fulltime/86400.);
-			    CHECK_SPACE(incr);
-			    break;
-			}
-			/* Set flag in case hours come next */
-			if (fulltime < 0) {
-			    CHECK_SPACE(1);	/* the minus sign */
-			    *s++ = '-';
-			    l++;
-			}
-			sign_printed = TRUE;
-			/* +/- integral day truncated toward zero */
-			sprintf(s, "%0*d", w, (int)floor(fabs(fulltime/86400.)));
-
-			/* Subtract the day component from the total */
-			fulltime -= sgn(fulltime) * 86400. * floor(fabs(fulltime/86400.));
-			break;
 		    case 'H':
 			/* +/- fractional hours (not wrapped at 24h) */
 			if (p > 0) {
-			    incr = snprintf(s, bsz-l-1, "%*.*f", w, p,
-				sign_printed ? fabs(fulltime)/3600. : fulltime/3600.);
+			    incr = snprintf(s, bsz-l-1, "%*.*f", w, p, fulltime/3600.);
 			    CHECK_SPACE(incr);
 			    break;
 			}
 			/* Set flag in case minutes come next */
-			if (fulltime < 0 && !sign_printed) {
+			if (fulltime < 0) {
 			    CHECK_SPACE(1);	/* the minus sign */
+			    sign_printed = TRUE;
 			    *s++ = '-';
 			    l++;
 			}
-			sign_printed = TRUE;
 			/* +/- integral hour truncated toward zero */
 			sprintf(s, "%0*d", w, (int)floor(fabs(fulltime/3600.)));
-
-			/* Subtract the hour component from the total */
-			fulltime -= sgn(fulltime) * 3600. * floor(fabs(fulltime/3600.));
 			break;
 		    case 'M':
 			/* +/- fractional minutes (not wrapped at 60m) */
@@ -683,20 +557,19 @@ xstrftime(
 			    break;
 			}
 			/* +/- integral minute truncated toward zero */
-			tminute = floor((fabs(fulltime/60.)));
-			if (fulltime < 0 && !sign_printed) {
+			tminute = floor(60. * (fabs(fulltime/3600.) - floor(fabs(fulltime/3600.))));
+			if (fulltime < 0) {
+			    if (!sign_printed) {
+				sign_printed = TRUE;
 				*s++ = '-';
 				l++;
+			    }
 			}
-			sign_printed = TRUE;
 			FORMAT_STRING(1, 2, tminute);	/* %02d */
-
-			/* Subtract the minute component from the total */
-			fulltime -= sgn(fulltime) * 60. * floor(fabs(fulltime/60.));
 			break;
 		    case 'S':
 			/* +/- fractional seconds */
-			tsecond = floor(fabs(fulltime));
+			tsecond = floor(60. * (fabs(fulltime/60.) - floor(fabs(fulltime/60.))));
 			if (fulltime < 0) {
 			    if (usec > 0)
 				usec = 1.0 - usec;
@@ -805,7 +678,7 @@ double
 gtimegm(struct tm *tm)
 {
     int i;
-    /* returns sec from year ZERO_YEAR in UTC, defined in gp_time.h */
+    /* returns sec from year ZERO_YEAR, defined in gp_time.h */
     double dsec = 0.;
 
     if (tm->tm_year < ZERO_YEAR) {
@@ -833,13 +706,8 @@ gtimegm(struct tm *tm)
     dsec *= 60.0;
     dsec += tm->tm_sec;
 
-#ifdef HAVE_STRUCT_TM_TM_GMTOFF
-    dsec -= tm->tm_gmtoff;
-
-    FPRINTF((stderr, "broken-down time : %02d/%02d/%d:%02d:%02d:%02d:(%02d:%02d) = %g seconds\n",
-	     tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour,
-	     tm->tm_min, tm->tm_sec, tm->tm_gmtoff / 3600, (tm->tm_gmtoff % 3600) / 60, dsec));
-#endif
+    FPRINTF((stderr, "broken-down time : %02d/%02d/%d:%02d:%02d:%02d = %g seconds\n", tm->tm_mday, tm->tm_mon + 1, tm->tm_year, tm->tm_hour,
+	     tm->tm_min, tm->tm_sec, dsec));
 
     return (dsec);
 }
@@ -861,9 +729,14 @@ ggmtime(struct tm *tm, double l_clock)
 
     tm->tm_year = ZERO_YEAR;
     tm->tm_mday = tm->tm_yday = tm->tm_mon = tm->tm_hour = tm->tm_min = tm->tm_sec = 0;
-    init_timezone(tm);
-
-    if (l_clock >= 0) {
+    if (l_clock < 0) {
+	while (l_clock < 0) {
+	    int days_in_year = gdysize(--tm->tm_year);
+	    l_clock += days_in_year * DAY_SEC;	/* 24*3600 */
+	    /* adding 371 is noop in modulo 7 arithmetic, but keeps wday +ve */
+	    wday += 371 - days_in_year;
+	}
+    } else {
 	for (;;) {
 	    int days_in_year = gdysize(tm->tm_year);
 	    if (l_clock < days_in_year * DAY_SEC)
@@ -873,13 +746,6 @@ ggmtime(struct tm *tm, double l_clock)
 	    /* only interested in result modulo 7, but %7 is expensive */
 	    wday += (days_in_year - 364);
 	}
-    } else {
-	while (l_clock < 0) {
-	    int days_in_year = gdysize(--tm->tm_year);
-	    l_clock += days_in_year * DAY_SEC;	/* 24*3600 */
-	    /* adding 371 is noop in modulo 7 arithmetic, but keeps wday +ve */
-	    wday += 371 - days_in_year;
-	}
     }
     tm->tm_yday = (int) (l_clock / DAY_SEC);
     l_clock -= tm->tm_yday * DAY_SEC;
@@ -897,12 +763,6 @@ ggmtime(struct tm *tm, double l_clock)
     while (days >= (i = mndday[tm->tm_mon] + (tm->tm_mon == 1 && (gdysize(tm->tm_year) > 365)))) {
 	days -= i;
 	tm->tm_mon++;
-	/* This catches round-off error that initially assigned a date to year N-1 */
-	/* but counting out seconds puts it in the first second of Jan year N.     */
-	if (tm->tm_mon > 11) {
-	    tm->tm_mon = 0;
-	    tm->tm_year++;
-	}
     }
     tm->tm_mday = days + 1;
 
@@ -911,3 +771,105 @@ ggmtime(struct tm *tm, double l_clock)
     return (0);
 }
 
+
+
+
+#else /* USE_SYSTEM_TIME */
+
+/* define gnu time routines in terms of system time routines */
+
+size_t
+gstrftime(char *buf, size_t bufsz, const char *fmt, double l_clock)
+{
+    time_t t = (time_t) l_clock;
+    return strftime(buf, bufsz, fmt, gmtime(&t));
+}
+
+double
+gtimegm(struct tm *tm)
+{
+    return (double) mktime(tm);
+}
+
+int
+ggmtime(struct tm *tm, double l_clock)
+{
+    time_t t = (time_t) l_clock;
+    struct tm *m = gmtime(&t);
+    *tm = *m;			/* can any non-ansi compilers not do this ? */
+    return 0;
+}
+
+/* supplemental routine gstrptime() to read a formatted time */
+
+char *
+gstrptime(char *s, char *fmt, struct tm *tm)
+{
+    FPRINTF((stderr, "gstrptime(\"%s\", \"%s\")\n", s, fmt));
+
+    /* linux does not appear to like years before 1902
+     * NT complains if its before 1970
+     * initialise fields to midnight, 1st Jan, 1970 (for relative times)
+     */
+    tm->tm_sec = tm->tm_min = tm->tm_hour = 0;
+    tm->tm_mday = 1;
+    tm->tm_mon = 0;
+    tm->tm_year = 70;
+    /* oops - it goes wrong without this */
+    tm->tm_isdst = 0;
+
+    for (; *fmt && *s; ++fmt) {
+	if (*fmt != '%') {
+	    if (*s != *fmt)
+		return s;
+	    ++s;
+	    continue;
+	}
+	assert(*fmt == '%');
+
+	switch (*++fmt) {
+	case 0:
+	    /* uh oh - % is last character in format */
+	    return s;
+	case '%':
+	    /* literal % */
+	    if (*s++ != '%')
+		return s - 1;
+	    continue;
+
+#define NOTHING	/* nothing */
+#define LETTER(L, width, field, extra)		\
+	case L:					\
+	    s=read_int(s,width,&tm->field);	\
+	    extra;				\
+	    continue;
+
+	    LETTER('d', 2, tm_mday, NOTHING);
+	    LETTER('m', 2, tm_mon, NOTHING);
+	    LETTER('y', 2, tm_year, NOTHING);
+	    LETTER('Y', 4, tm_year, tm->tm_year -= 1900);
+	    LETTER('H', 2, tm_hour, NOTHING);
+	    LETTER('M', 2, tm_min, NOTHING);
+	    LETTER('S', 2, tm_sec, NOTHING);
+#undef NOTHING
+#undef LETTER
+
+	default:
+	    int_error(DATAFILE, "incorrect time format character");
+	}
+    }
+
+    FPRINTF((stderr, "Before mktime : %d/%d/%d:%d:%d:%d\n", tm->tm_mday, tm->tm_mon, tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec));
+    /* mktime range-checks the time */
+
+    if (mktime(tm) == -1) {
+	FPRINTF((stderr, "mktime() was not happy\n"));
+	int_error(DATAFILE, "Invalid date/time [mktime() did not like it]");
+    }
+    FPRINTF((stderr, "After mktime : %d/%d/%d:%d:%d:%d\n", tm->tm_mday, tm->tm_mon, tm->tm_year, tm->tm_hour, tm->tm_min, tm->tm_sec));
+
+    return s;
+}
+
+
+#endif /* USE_SYSTEM_TIME */
